import unittest
import mock
import logging
import os
import esgfpid.rabbit.connparams
LOGGER = logging.getLogger(__name__)
LOGGER.addHandler(logging.NullHandler())

class ThreadBuilderTestCase(unittest.TestCase):

    def setUp(self):
        LOGGER.info('######## Next test ########## (%s)' % os.path.basename(__file__))

    def tearDown(self):
        LOGGER.info('#############################')

    def make_builder(self, args=None):

        # Mocked:
        self.cred = esgfpid.rabbit.connparams.get_credentials('userfoo', 'pwfoo')
        if args is None:
            args = {
                "urls_fallback":['bar.de', 'baz.de'],
                "url_preferred":'foo.de',
                "exchange_name":'fooexchange'
            }
        args['credentials'] = self.cred
        self.thread = mock.MagicMock()
        self.feeder = mock.MagicMock()
        self.acceptor = mock.MagicMock()
        self.shutter = mock.MagicMock()
        self.statemachine = esgfpid.rabbit.asynchronous.thread_statemachine.StateMachine()
        self.confirmer = esgfpid.rabbit.asynchronous.thread_confirmer.ConfirmReactor()
        builder = esgfpid.rabbit.asynchronous.thread_builder.ConnectionBuilder(
            self.thread,
            self.statemachine,
            self.confirmer,
            self.feeder,
            self.acceptor,
            self.shutter,
            args)
        return builder

    def add_leftovers_to_confirmer(self):
        self.confirmer._ConfirmReactor__unconfirmed_delivery_tags = copy.copy(UNCONFIRMED_TAGS)
        self.confirmer._ConfirmReactor__unconfirmed_messages_dict = copy.copy(UNCONFIRMED_MESSAGES)

    # Tests

    #
    # Publish a message
    #

    @mock.patch('esgfpid.rabbit.asynchronous.thread_builder.ConnectionBuilder._ConnectionBuilder__please_open_connection')
    def test_build_connection_1_ok(self, connpatch):

        # Patch
        connpatch.return_value = mock.MagicMock()

        # Preparation:
        builder = self.make_builder()

        # Run code to be tested:
        builder.trigger_connection_to_rabbit_etc()
        # Tests:
        # trigger_connection_to_rabbit_etc
        # __connect_to_rabbit
        # __get_connection_params

        # Check result:
        self.assertEquals(connpatch.call_count, 1)

    @mock.patch('esgfpid.rabbit.asynchronous.thread_builder.ConnectionBuilder._ConnectionBuilder__please_open_rabbit_channel')
    def test_build_connection_2_on_connection_open_ok(self, channelpatch):

        # Patch
        channelpatch.return_value = mock.MagicMock()

        # Preparation:
        builder = self.make_builder()

        # Run code to be tested:
        builder.on_connection_open(None)
        # Tests:
        # on_connection_open
        # __add_on_connection_close_callback

        # Check result:
        self.assertEquals(channelpatch.call_count, 1)

    def test_build_connection_3_on_channel_open_ok(self):

        # Preparation:
        builder = self.make_builder()
        testchannel = mock.MagicMock()
        self.statemachine.set_to_waiting_to_be_available()

        # Run code to be tested:
        builder.on_channel_open(testchannel)
        # Tests:
        # on_channel_open
        # __add_on_channel_close_callback
        # __add_on_return_callback
        # __make_channel_confirm_delivery
        # __make_ready_for_publishing
        # __check_for_already_arrived_messages

        # Check result:
        self.assertTrue(self.statemachine.is_available_for_client_publishes())
        self.acceptor.trigger_publishing_n_messages_if_ok.assert_not_called()

    def test_build_connection_3_on_channel_open_messages_waiting_ok(self):

        # Preparation:
        builder = self.make_builder()
        testchannel = mock.MagicMock()
        self.statemachine.set_to_waiting_to_be_available()
        self.feeder.get_num_unpublished = mock.MagicMock(return_value=10)

        # Run code to be tested:
        builder.on_channel_open(testchannel)
        # Tests:
        # on_channel_open
        # __add_on_channel_close_callback
        # __add_on_return_callback
        # __make_channel_confirm_delivery
        # __make_ready_for_publishing
        # __check_for_already_arrived_messages

        # Check result:
        self.assertTrue(self.statemachine.is_available_for_client_publishes())
        self.acceptor.trigger_publishing_n_messages_if_ok.assert_called_with(10)

    def test_build_connection_3_on_channel_open_forceclosed(self):

        # Preparation:
        builder = self.make_builder()
        testchannel = mock.MagicMock()
        self.statemachine.closed_by_publisher = True
        self.statemachine.set_to_permanently_unavailable()

        # Run code to be tested:
        builder.on_channel_open(testchannel)
        # Tests:
        # on_channel_open
        # __add_on_channel_close_callback
        # __add_on_return_callback
        # __make_channel_confirm_delivery
        # __make_ready_for_publishing
        # __reclose_freshly_made_connection
        # __safety_finish

        # Check result:
        self.assertTrue(self.statemachine.is_permanently_unavailable())
        self.assertTrue(self.statemachine.closed_by_publisher)
        self.acceptor.trigger_publishing_n_messages_if_ok.assert_not_called()
        self.shutter.safety_finish.assert_called_with('closed before connection was ready. reclosing.')

    def test_build_connection_3_on_channel_open_forceclosed_pending_messages(self):

        # Preparation:
        builder = self.make_builder()
        testchannel = mock.MagicMock()
        self.statemachine.closed_by_publisher = True
        self.statemachine.set_to_permanently_unavailable()
        self.feeder.get_num_unpublished = mock.MagicMock(return_value=10)

        # Run code to be tested:
        builder.on_channel_open(testchannel)
        # Tests:
        # on_channel_open
        # __add_on_channel_close_callback
        # __add_on_return_callback
        # __make_channel_confirm_delivery
        # __make_ready_for_publishing
        # __reclose_freshly_made_connection
        # __safety_finish

        # Check result:
        self.assertTrue(self.statemachine.is_permanently_unavailable())
        self.acceptor.trigger_publishing_n_messages_if_ok.assert_not_called()
        self.shutter.safety_finish.assert_called_with('closed before connection was ready. reclosing.')

    def test_build_connection_3_on_channel_open_gentlyclosed(self):

        # Preparation:
        builder = self.make_builder()
        testchannel = mock.MagicMock()
        self.statemachine.set_to_wanting_to_stop()

        # Run code to be tested:
        builder.on_channel_open(testchannel)
        # Tests:
        # on_channel_open
        # __add_on_channel_close_callback
        # __add_on_return_callback
        # __make_channel_confirm_delivery
        # __make_ready_for_publishing
        # __check_for_already_arrived_messages

        # Check result:
        self.assertTrue(self.statemachine.is_available_but_wants_to_stop())
        self.acceptor.trigger_publishing_n_messages_if_ok.assert_not_called()
        self.shutter.safety_finish.assert_not_called()

    def test_build_connection_3_on_channel_open_gentlyclosed_pending(self):

        # Preparation:
        builder = self.make_builder()
        testchannel = mock.MagicMock()
        self.statemachine.set_to_wanting_to_stop()
        self.feeder.get_num_unpublished = mock.MagicMock(return_value=10)

        # Run code to be tested:
        builder.on_channel_open(testchannel)
        # Tests:
        # on_channel_open
        # __add_on_channel_close_callback
        # __add_on_return_callback
        # __make_channel_confirm_delivery
        # __make_ready_for_publishing
        # __check_for_already_arrived_messages

        # Check result:
        self.assertTrue(self.statemachine.is_available_but_wants_to_stop())
        self.acceptor.trigger_publishing_n_messages_if_ok.assert_called_with(10)
        self.shutter.safety_finish.assert_not_called()

    def test_on_connection_error_none_left(self):

        # Preparation:
        args = {
            "urls_fallback":[],
            "url_preferred":'foo.de',
            "exchange_name":'fooexchange'
        }
        builder = self.make_builder(args)
        # To avoid that it tries to reconnect some more times:
        builder._ConnectionBuilder__reconnect_counter = esgfpid.defaults.RABBIT_ASYN_RECONNECTION_MAX_TRIES
        self.statemachine.set_to_waiting_to_be_available()

        # Run code to be tested:
        connection = mock.MagicMock()
        builder.on_connection_error(connection, 'foo message')
        # Tests:
        # on_connection_error
        # __is_fallback_url_left
        # __change_state_to_permanently_could_not_connect
        # __inform_permanently_could_not_connect

        # Check result:
        self.assertTrue(self.statemachine.is_permanently_unavailable())
        self.assertTrue(self.statemachine.could_not_connect)
        self.acceptor.trigger_publishing_n_messages_if_ok.assert_not_called()

    @mock.patch('esgfpid.rabbit.asynchronous.thread_builder.ConnectionBuilder._ConnectionBuilder__please_open_connection')
    def test_on_connection_error_urls_left(self, pleasepatch):

        # Preparation:
        builder = self.make_builder()
        args = {
            "urls_fallback":['foobarbaz'],
            "url_preferred":'foo.de',
            "exchange_name":'fooexchange'
        }
        builder = self.make_builder(args)
        #builder.RABBIT_HOSTS = ['foobarbaz']
        pleasepatch.return_value = 'mock_connection'
        self.statemachine.set_to_waiting_to_be_available()

        # Run code to be tested:
        builder.on_connection_error(None, 'foo message')
        # Tests:
        # on_connection_error
        # __is_fallback_url_left
        # __change_state_to_permanently_could_not_connect
        # __inform_permanently_could_not_connect

        # Check result:
        self.assertEquals('mock_connection', self.thread._connection)
        self.assertEquals(builder.CURRENT_HOST, 'foobarbaz')
        self.assertTrue(self.statemachine.is_waiting_to_be_available())
        self.acceptor.trigger_publishing_n_messages_if_ok.assert_not_called()


    #
    # on_channel_close and on_connection_close
    #

    def test_on_channel_closed_unexpected(self):

        # Preparation:
        builder = self.make_builder()
        self.statemachine.set_to_permanently_unavailable()
        self.statemachine.closed_by_publisher = True

        # Run code to be tested:
        connection = mock.MagicMock()
        builder.on_channel_closed(connection, 111, 'foo')

        # Check result
        self.thread._connection.close.assert_not_called()

    def test_on_channel_closed_ok(self):

        # Preparation:
        builder = self.make_builder()
        self.statemachine.set_to_available()
        self.statemachine.closed_by_publisher = False
        self.thread._connection = mock.MagicMock()
        self.thread._connection.close.return_value = mock.MagicMock()

        # Run code to be tested:
        builder.on_channel_closed(None, 111, 'foo')

        # Check result
        self.thread._connection.close.assert_called_with()

    def test_on_connection_closed_gentle_ok(self):

        # Preparation:
        builder = self.make_builder()
        self.statemachine.set_to_available()
        self.statemachine.closed_by_publisher = False
        self.thread.ERROR_CODE_CONNECTION_CLOSED_BY_USER=999
        self.thread.ERROR_TEXT_CONNECTION_FORCE_CLOSED='(forced finish)'
        self.thread.ERROR_TEXT_CONNECTION_NORMAL_SHUTDOWN='(not reopen)'
        #self.thread._connection = mock.MagicMock()
        #self.thread._connection.ioloop = mock.MagicMock()
        #self.thread._connection.ioloop.stop.return_value = mock.MagicMock()
        #self.thread._connection.close.return_value = mock.MagicMock()

        # Run code to be tested:
        builder.on_connection_closed(None, 999, 'foo(not reopen)foo')

        # Check result
        self.thread._connection.ioloop.stop.assert_called_with()
        self.assertTrue(self.statemachine.is_permanently_unavailable())

    def test_on_connection_closed_forced_ok(self):

        # Preparation:
        builder = self.make_builder()
        self.statemachine.set_to_available()
        self.statemachine.closed_by_publisher = False
        self.thread.ERROR_CODE_CONNECTION_CLOSED_BY_USER=999
        self.thread.ERROR_TEXT_CONNECTION_FORCE_CLOSED='(forced finish)'
        self.thread.ERROR_TEXT_CONNECTION_NORMAL_SHUTDOWN='(not reopen)'
        #self.thread._connection.ioloop.stop.return_value = mock.MagicMock()

        # Run code to be tested:
        builder.on_connection_closed(None, 999, 'foo(forced finish)foo')

        # Check result
        self.thread._connection.ioloop.stop.assert_called_with()
        self.assertTrue(self.statemachine.is_permanently_unavailable())


    def test_on_connection_closed_unexpected_ok(self):

        # Preparation:
        builder = self.make_builder()
        self.statemachine.set_to_available()
        self.statemachine.closed_by_publisher = False
        self.thread.ERROR_CODE_CONNECTION_CLOSED_BY_USER=999
        self.thread.ERROR_TEXT_CONNECTION_FORCE_CLOSED='(forced finish)'
        self.thread.ERROR_TEXT_CONNECTION_NORMAL_SHUTDOWN='(not reopen)'

        # Run code to be tested:
        builder.on_connection_closed(self.thread._connection, 111, '(foo)')

        # Check result
        self.thread._connection.ioloop.stop.assert_not_called()
        self.assertTrue(self.statemachine.is_waiting_to_be_available())
        self.thread._connection.add_timeout.assert_called_with(5, builder.reconnect)

    #
    # reconnect
    #

    @mock.patch('esgfpid.rabbit.asynchronous.thread_builder.ConnectionBuilder._ConnectionBuilder__connect_to_rabbit')
    def test_reconnect_no_rescued_messages(self, connpatch):

        # Preparation:
        builder = self.make_builder()

        # Run code to be tested:
        builder.reconnect()
        # Tests:


        # Check result:
        self.feeder.reset_message_number.assert_called_with()
        self.acceptor.send_many_messages.assert_not_called()
        self.assertEquals(connpatch.call_count, 1)
        builder.thread._connection.ioloop.start.assert_any_call()
        builder.thread._connection.ioloop.stop.assert_any_call()


    @mock.patch('esgfpid.rabbit.asynchronous.thread_builder.ConnectionBuilder._ConnectionBuilder__connect_to_rabbit')
    def test_reconnect_some_rescued_messages(self, connpatch):

        # Preparation:
        builder = self.make_builder()
        self.confirmer.put_to_unconfirmed_delivery_tags(1)
        self.confirmer.put_to_unconfirmed_delivery_tags(2)
        self.confirmer.put_to_unconfirmed_delivery_tags(3)
        self.confirmer.put_to_unconfirmed_messages_dict(1,'foo1')
        self.confirmer.put_to_unconfirmed_messages_dict(2,'foo2')
        self.confirmer.put_to_unconfirmed_messages_dict(3,'foo3')

        # Run code to be tested:
        builder.reconnect()

        # Check result:
        self.feeder.reset_message_number.assert_called_with()
        self.acceptor.send_many_messages.assert_called_with(['foo1','foo3','foo2'])
        self.assertEquals(connpatch.call_count, 1)
        builder.thread._connection.ioloop.start.assert_any_call()
        builder.thread._connection.ioloop.stop.assert_any_call()

    #
    # ioloop
    #

    def test_start_ioloop_immediately(self):

        # Preparation:
        builder = self.make_builder()
        builder.thread._connection = mock.MagicMock()
        builder.thread._connection.is_open = True # will succeed

        # Run code to be tested:
        builder.start_ioloop_when_connection_ready(20,20)

        # Check result:
        builder.thread._connection.ioloop.start.assert_any_call()

    def test_start_ioloop_give_up(self):

        # Preparation:
        builder = self.make_builder()
        self.statemachine.could_not_connect = True # will give up!
        self.statemachine.set_to_permanently_unavailable()
        builder.thread._connection = mock.MagicMock()
        builder.thread._connection.is_open = True

        # Run code to be tested:
        builder.start_ioloop_when_connection_ready(20,20)

        # Check result:
        builder.thread._connection.ioloop.start.assert_not_called()

    def test_start_ioloop_second_try(self):

        # Preparation:
        builder = self.make_builder()
        builder.thread._connection.side_effect = [None, mock.MagicMock]
        #builder.thread._connection.is_open = []

        # Run code to be tested:
        builder.start_ioloop_when_connection_ready(20,20)

        # Check result:
        builder.thread._connection.ioloop.start.assert_any_call()


    def test_start_ioloop_loop_on(self):

        # Preparation:
        builder = self.make_builder()
        builder.thread._connection = None

        # Run code to be tested:
        with self.assertRaises(ValueError):
            builder.start_ioloop_when_connection_ready(3,0.1)




TESTVALUES = dict(
    solr_url='http://solr.foo',
    prefix='21.14foo',
    drs_id1='my/drs/id',
    version_number1=201603,
    suffix1='afd65cd0-9296-35bc-a706-be98665c9c36',
    data_node='esgf-foo.dkrz.de',
    url_rabbit_trusted='http://trusted.rabbit.foo',
    url_rabbit_open='http://rabbit.foo',
    rabbit_exchange_name='exch', # TODO JUST ONE NAME!
    thredds_service_path='test/thr/path',
    rabbit_username_open='RogerRabbit',
    rabbit_username_trusted='RogerRabbit',
    rabbit_password='carrotDreams',
    errata_ids=['123456','654321'],
    errata_id='123456',
    file_name1='mytestfilename.nc',
    file_suffix1='xyz123abc',
    file_suffix2='xyz123def',
    checksum1='testchecksumabc',
    file_size1=10010,
    publish_path1='my/publish/path',
    checksum_type1='SHA999',
    file_version1='987'
)




import unittest
import argparse
import logging
import datetime
import os
import sys
sys.path.append("..")
import esgfpid

# Setup logging:
path = 'logs'
logdate = datetime.datetime.now().strftime('%Y-%m-%d_%H:%M:%S')
filename = path+'/test_log_'+logdate+'.log'
esgfpid.utils.ensure_directory_exists(path)
logging.basicConfig(level=logging.DEBUG, filename=filename, filemode='w')
pikalogger = logging.getLogger('pika')
pikalogger.setLevel(logging.INFO)
#pikalogger.propagate = False


if __name__ == '__main__':
    desc = ('Test script for esgfpid library, including unit tests and integration '+
           'tests.')
    
    # Argument parsing
    parser = argparse.ArgumentParser(description=desc)
    
    # Unit or integration?
    parser.add_argument('-u','--unit', dest='unit',
                   help=('Run the unit tests?'),
                   action='store_true')
    parser.add_argument('-i','--integration', dest='inte',
                   help=('Run the integration tests?'),
                   action='store_true')
    parser.set_defaults(unit=False)
    parser.set_defaults(inte=False)

    # Modules to be tested:
    parser.add_argument('-m','--modules', metavar='mod', nargs='*',
                   help=('Which modules to test. '+
                         'Possible values: "all", "solr", "rabbit", "publish", "unpublish", '+
                         '"errata", "utils", "api", "check", "messages", "consistency", "shopping_cart"). '+
                         'Defaults to "all".'),
                   default=['all'], action='store')

    # Synchron or asynchron?
    parser.add_argument('-a','--asynchron', dest='asyn',
                   help=('Run the asynchronous rabbit tests?'),
                   action='store_true')
    parser.add_argument('-s','--synchron', dest='asyn',
                   help=('Run the synchronous rabbit tests?'),
                   action='store_false')
    parser.set_defaults(asyn=True)

    param = parser.parse_args()
    #print('Specified test types: '+str(param.testtype))
    print('Modules to be tested: '+ ', '.join(param.modules))

    # Collect tests:
    verbosity = 5
    descriptions = 0
    print '\nCollecting tests:'
    tests_to_run = []
    numtests = 0

    if param.unit:
    
        if 'utils' in param.modules or 'all' in param.modules:

            from testcases.util_tests import UtilsTestCase
            tests = unittest.TestLoader().loadTestsFromTestCase(UtilsTestCase)
            tests_to_run.append(tests)
            n = tests.countTestCases()
            numtests += n

            from testcases.util_logging_tests import UtilsLoggingTestCase
            tests = unittest.TestLoader().loadTestsFromTestCase(UtilsLoggingTestCase)
            tests_to_run.append(tests)
            n = tests.countTestCases()
            numtests += n

            from testcases.rabbit_rabbitutil_tests import RabbitUtilsTestCase
            tests = unittest.TestLoader().loadTestsFromTestCase(RabbitUtilsTestCase)
            tests_to_run.append(tests)
            n = tests.countTestCases()
            numtests += n

            from testcases.errormessage_util_tests import ErrorMessageUtilsTestCase
            tests = unittest.TestLoader().loadTestsFromTestCase(ErrorMessageUtilsTestCase)
            tests_to_run.append(tests)
            n = tests.countTestCases()
            numtests += n

        if 'messages' in param.modules or 'all' in param.modules:

            from testcases.messages_tests import MessageCreationTestcase
            tests = unittest.TestLoader().loadTestsFromTestCase(MessageCreationTestcase)
            tests_to_run.append(tests)
            n = tests.countTestCases()
            numtests += n

        if 'check' in param.modules or 'all' in param.modules:

            pass
            # TODO
            #from testcases.check_tests import CheckTestCase
            #tests = unittest.TestLoader().loadTestsFromTestCase(CheckTestCase)
            #tests_to_run.append(tests)
            #n = tests.countTestCases()
            #numtests += n

        if 'shopping_cart' in param.modules or 'all' in param.modules:

            from testcases.shopping_cart_tests import ShoppingCartTestCase
            tests = unittest.TestLoader().loadTestsFromTestCase(ShoppingCartTestCase)
            tests_to_run.append(tests)
            n = tests.countTestCases()
            numtests += n

        if 'solr' in param.modules or 'all' in param.modules:

            from testcases.solr_utils_tests import SolrUtilsTestCase
            tests = unittest.TestLoader().loadTestsFromTestCase(SolrUtilsTestCase)
            tests_to_run.append(tests)
            n = tests.countTestCases()
            numtests += n

            from testcases.solr_tests import SolrTestCase
            tests = unittest.TestLoader().loadTestsFromTestCase(SolrTestCase)
            tests_to_run.append(tests)
            n = tests.countTestCases()
            numtests += n

            from testcases.solr_task1_tests import SolrTask1TestCase
            tests = unittest.TestLoader().loadTestsFromTestCase(SolrTask1TestCase)
            tests_to_run.append(tests)
            n = tests.countTestCases()
            numtests += n

            from testcases.solr_task2_tests import SolrTask2TestCase
            tests = unittest.TestLoader().loadTestsFromTestCase(SolrTask2TestCase)
            tests_to_run.append(tests)
            n = tests.countTestCases()
            numtests += n

            from testcases.solr_server_tests import SolrServerConnectorTestCase
            tests = unittest.TestLoader().loadTestsFromTestCase(SolrServerConnectorTestCase)
            tests_to_run.append(tests)
            n = tests.countTestCases()
            numtests += n

        if 'rabbit' in param.modules or 'all' in param.modules:

            if param.asyn:

                # TODO
                
                #from testcases.rabbit_rabbit_asyn_tests import RabbitTestCase
                #tests = unittest.TestLoader().loadTestsFromTestCase(RabbitTestCase)
                #tests_to_run.append(tests)
                #numtests += tests.countTestCases()

                #from testcases.rabbit_asynchronous_asynchronous_tests import RabbitAsynConnectorTestCase
                #tests = unittest.TestLoader().loadTestsFromTestCase(RabbitAsynConnectorTestCase)
                #tests_to_run.append(tests)
                #numtests += tests.countTestCases()

                #from testcases.rabbit_thread_confirmer_tests import ThreadConfirmerTestCase
                #tests = unittest.TestLoader().loadTestsFromTestCase(ThreadConfirmerTestCase)
                #tests_to_run.append(tests)
                #numtests += tests.countTestCases()

                #from testcases.rabbit_thread_acceptor_tests import ThreadAcceptorTestCase
                #tests = unittest.TestLoader().loadTestsFromTestCase(ThreadAcceptorTestCase)
                #tests_to_run.append(tests)
                #numtests += tests.countTestCases()

                #from testcases.rabbit_thread_shutter_tests import ThreadShutterTestCase
                #tests = unittest.TestLoader().loadTestsFromTestCase(ThreadShutterTestCase)
                #tests_to_run.append(tests)
                #numtests += tests.countTestCases()

                #from testcases.rabbit_thread_feeder_tests import ThreadFeederTestCase
                #tests = unittest.TestLoader().loadTestsFromTestCase(ThreadFeederTestCase)
                #tests_to_run.append(tests)
                #numtests += tests.countTestCases()

                #from testcases.rabbit_thread_builder_tests import ThreadBuilderTestCase
                #tests = unittest.TestLoader().loadTestsFromTestCase(ThreadBuilderTestCase)
                #tests_to_run.append(tests)
                #numtests += tests.countTestCases()

                from testcases.rabbit_asynchronous_asynchronous_module_tests import RabbitAsynModuleTestCase
                tests = unittest.TestLoader().loadTestsFromTestCase(RabbitAsynModuleTestCase)
                tests_to_run.append(tests)
                numtests += tests.countTestCases()

            else:

                from testcases.rabbit_rabbit_syn_tests import RabbitTestCase
                tests = unittest.TestLoader().loadTestsFromTestCase(RabbitTestCase)
                tests_to_run.append(tests)
                n = tests.countTestCases()
                numtests += n

                from testcases.rabbit_synchronous_tests import RabbitConnectorTestCase
                tests = unittest.TestLoader().loadTestsFromTestCase(RabbitConnectorTestCase)
                tests_to_run.append(tests)
                n = tests.countTestCases()
                numtests += n

        if 'errata' in param.modules or 'all' in param.modules:

            from testcases.errata_tests import ErrataTestCase
            tests = unittest.TestLoader().loadTestsFromTestCase(ErrataTestCase)
            tests_to_run.append(tests)
            n = tests.countTestCases()
            numtests += n

        if 'unpublish' in param.modules or 'all' in param.modules:

            from testcases.unpublication_tests import UnpublicationTestCase
            tests = unittest.TestLoader().loadTestsFromTestCase(UnpublicationTestCase)
            tests_to_run.append(tests)
            n = tests.countTestCases()
            numtests += n

        if 'publish' in param.modules or 'all' in param.modules:

            from testcases.publish_tests import PublishTestCase
            tests = unittest.TestLoader().loadTestsFromTestCase(PublishTestCase)
            tests_to_run.append(tests)
            n = tests.countTestCases()
            numtests += n

        if 'api' in param.modules or 'all' in param.modules:

            from testcases.api_tests import ApiTestCase
            tests = unittest.TestLoader().loadTestsFromTestCase(ApiTestCase)
            tests_to_run.append(tests)
            n = tests.countTestCases()
            numtests += n

        if 'consistency' in param.modules or 'all' in param.modules:

            pass
            # TODO
            #from testcases.consistency_tests import ConsistencyTestCase
            #tests = unittest.TestLoader().loadTestsFromTestCase(ConsistencyTestCase)
            #tests_to_run.append(tests)
            #n = tests.countTestCases()
            #numtests += n

    if param.inte:

        if 'rabbit' in param.modules or 'all' in param.modules:

            if param.asyn:
                from testcases.integration_rabbit_tests_asynchron import RabbitIntegrationTestCase
            else:
                from testcases.integration_rabbit_tests_synchron import RabbitIntegrationTestCase

            tests = unittest.TestLoader().loadTestsFromTestCase(RabbitIntegrationTestCase)
            tests_to_run.append(tests)
            n = tests.countTestCases()
            numtests += n

        if 'solr' in param.modules or 'all' in param.modules:

            from testcases.integration_solr_tests import SolrIntegrationTestCase
            tests = unittest.TestLoader().loadTestsFromTestCase(SolrIntegrationTestCase)
            tests_to_run.append(tests)
            n = tests.countTestCases()
            numtests += n

            from testcases.integration_solr_server_tests import SolrServerIntegrationTestCase
            tests = unittest.TestLoader().loadTestsFromTestCase(SolrServerIntegrationTestCase)
            tests_to_run.append(tests)
            n = tests.countTestCases()
            numtests += n

            from testcases.integration_solr_task1_tests import SolrTask1IntegrationTestCase
            tests = unittest.TestLoader().loadTestsFromTestCase(SolrTask1IntegrationTestCase)
            tests_to_run.append(tests)
            n = tests.countTestCases()
            numtests += n

    # Run all of them:
    print 'Run '+str(numtests)+' tests.'
    test_suites = unittest.TestSuite(tests_to_run)
    print '\nStarting tests:'
    logging.info("Starting tests:")
    test_result = unittest.TextTestRunner(descriptions=descriptions, verbosity=verbosity).run(test_suites)
    sys.exit(not test_result.wasSuccessful())

    # Run with:
    # python -m coverage run main_test_script.py
    # python -m coverage report



import unittest
import mock
import logging
import json
import esgfpid.assistant.errata
import tests.mocks.rabbitmock
import tests.mocks.solrmock
import tests.utils as utils
from tests.utils import compare_json_return_errormessage as error_message
from esgfpid.defaults import ROUTING_KEY_BASIS as ROUTING_KEY_BASIS


# Logging
LOGGER = logging.getLogger(__name__)
LOGGER.addHandler(logging.NullHandler())

# Test resources:
from resources.TESTVALUES import TESTVALUES as TESTVALUES

class ErrataTestCase(unittest.TestCase):

    def setUp(self):
        LOGGER.info('######## Next test (%s) ##########', __name__)

    def tearDown(self):
        LOGGER.info('#############################')

    def make_test_coupler_for_sending_messages(self):
        testcoupler = esgfpid.coupling.Coupler(
            handle_prefix = TESTVALUES['prefix'],
            messaging_service_urls_open = TESTVALUES['url_rabbit_open'],
            messaging_service_url_trusted = TESTVALUES['url_rabbit_trusted'],
            messaging_service_exchange_name = TESTVALUES['rabbit_exchange_name'],
            solr_url = TESTVALUES['solr_url'],
            messaging_service_username_open = TESTVALUES['rabbit_username_open'],
            messaging_service_username_trusted = TESTVALUES['rabbit_username_trusted'],
            messaging_service_password = TESTVALUES['rabbit_password'],
            solr_switched_off = True,
            solr_https_verify=False
        )
        # Replace objects that interact with servers with mocks
        testcoupler._Coupler__rabbit_message_sender = tests.mocks.rabbitmock.SimpleMockRabbitSender()
        return testcoupler

    def __get_received_message_from_rabbit_mock(self, coupler, index):
        return coupler._Coupler__rabbit_message_sender.received_messages[index]

    ### Actual test cases: ###

    def test_init_ok(self):

        # Test variables
        testcoupler = self.make_test_coupler_for_sending_messages()
        drs_id = TESTVALUES['drs_id1']
        version_number = TESTVALUES['version_number1']
        prefix = TESTVALUES['prefix']
        errata_id = TESTVALUES['errata_id']
  
        # Run code to be tested:
        assistant = esgfpid.assistant.errata.ErrataAssistant(
            prefix=prefix,
            coupler=testcoupler
        )

        # Check result:
        self.assertIsInstance(assistant, esgfpid.assistant.errata.ErrataAssistant, 'Constructor fail.')

    def test_add_errata_id_one_as_string(self):

        # Test variables
        testcoupler = self.make_test_coupler_for_sending_messages()
        drs_id = TESTVALUES['drs_id1']
        version_number = TESTVALUES['version_number1']
        prefix = TESTVALUES['prefix']
        errata_id = TESTVALUES['errata_id']
  
        # Preparations
        assistant = esgfpid.assistant.errata.ErrataAssistant(
            prefix=prefix,
            coupler=testcoupler
        )

        # Run code to be tested:
        assistant.add_errata_ids(
            drs_id=drs_id,
            version_number=version_number,
            errata_ids=errata_id
        )

        # Check result:
        expected_rabbit_task = {
            "handle": "hdl:"+prefix+'/'+TESTVALUES['suffix1'],
            "operation": "add_errata_ids",
            "message_timestamp":"anydate",
            "errata_ids":[errata_id],
            "ROUTING_KEY": ROUTING_KEY_BASIS+'errata.add',
            "drs_id":drs_id,
            "version_number":version_number
        }
        received_rabbit_task = self.__get_received_message_from_rabbit_mock(testcoupler, 0)
        same = utils.is_json_same(expected_rabbit_task, received_rabbit_task)
        self.assertTrue(same, error_message(expected_rabbit_task, received_rabbit_task))

    def test_add_errata_id_one_as_list(self):

        # Test variables
        testcoupler = self.make_test_coupler_for_sending_messages()
        drs_id = TESTVALUES['drs_id1']
        version_number = TESTVALUES['version_number1']
        prefix = TESTVALUES['prefix']
        errata_id = [TESTVALUES['errata_id']]
  
        # Preparations
        assistant = esgfpid.assistant.errata.ErrataAssistant(
            prefix=prefix,
            coupler=testcoupler
        )

        # Run code to be tested:
        assistant.add_errata_ids(
            drs_id=drs_id,
            version_number=version_number,
            errata_ids=errata_id
        )

        # Check result:
        expected_rabbit_task = {
            "handle": "hdl:"+prefix+'/'+TESTVALUES['suffix1'],
            "operation": "add_errata_ids",
            "message_timestamp":"anydate",
            "errata_ids":errata_id,
            "ROUTING_KEY": ROUTING_KEY_BASIS+'errata.add',
            "drs_id":drs_id,
            "version_number":version_number
        }
        received_rabbit_task = self.__get_received_message_from_rabbit_mock(testcoupler, 0)
        same = utils.is_json_same(expected_rabbit_task, received_rabbit_task)
        self.assertTrue(same, error_message(expected_rabbit_task, received_rabbit_task))

    def test_add_errata_id_several(self):

        # Test variables
        testcoupler = self.make_test_coupler_for_sending_messages()
        drs_id = TESTVALUES['drs_id1']
        version_number = TESTVALUES['version_number1']
        prefix = TESTVALUES['prefix']
        errata_ids = TESTVALUES['errata_ids']
  
        # Preparations
        assistant = esgfpid.assistant.errata.ErrataAssistant(
            prefix=prefix,
            coupler=testcoupler
        )

        # Run code to be tested:
        assistant.add_errata_ids(
            drs_id=drs_id,
            version_number=version_number,
            errata_ids=errata_ids
        )

        # Check result:
        expected_rabbit_task = {
            "handle": "hdl:"+prefix+'/'+TESTVALUES['suffix1'],
            "operation": "add_errata_ids",
            "message_timestamp":"anydate",
            "errata_ids":errata_ids,
            "ROUTING_KEY": ROUTING_KEY_BASIS+'errata.add',
            "drs_id":drs_id,
            "version_number":version_number
        }
        received_rabbit_task = self.__get_received_message_from_rabbit_mock(testcoupler, 0)
        same = utils.is_json_same(expected_rabbit_task, received_rabbit_task)
        self.assertTrue(same, error_message(expected_rabbit_task, received_rabbit_task))

    def test_add_errata_id_missing_args(self):

        # Test variables
        testcoupler = self.make_test_coupler_for_sending_messages()
        drs_id = TESTVALUES['drs_id1']
        version_number = TESTVALUES['version_number1']
        prefix = TESTVALUES['prefix']
        errata_ids = TESTVALUES['errata_ids']
  
        # Preparations
        assistant = esgfpid.assistant.errata.ErrataAssistant(
            prefix=prefix,
            coupler=testcoupler
        )

        # Run code to be tested and check exception:
        with self.assertRaises(esgfpid.exceptions.ArgumentError):
            assistant.add_errata_ids(
                drs_id=drs_id,
                version_number=version_number
            )

    def test_remove_errata_id_several(self):

        # Test variables
        testcoupler = self.make_test_coupler_for_sending_messages()
        drs_id = TESTVALUES['drs_id1']
        version_number = TESTVALUES['version_number1']
        prefix = TESTVALUES['prefix']
        errata_ids = TESTVALUES['errata_ids']
  
        # Preparations
        assistant = esgfpid.assistant.errata.ErrataAssistant(
            prefix=prefix,
            coupler=testcoupler
        )

        # Run code to be tested:
        assistant.remove_errata_ids(
            drs_id=drs_id,
            version_number=version_number,
            errata_ids=errata_ids
        )

        # Check result:
        expected_rabbit_task = {
            "handle": "hdl:"+prefix+'/'+TESTVALUES['suffix1'],
            "operation": "remove_errata_ids",
            "message_timestamp":"anydate",
            "errata_ids":errata_ids,
            "ROUTING_KEY": ROUTING_KEY_BASIS+'errata.remove',
            "drs_id":drs_id,
            "version_number":version_number
        }
        received_rabbit_task = self.__get_received_message_from_rabbit_mock(testcoupler, 0)
        same = utils.is_json_same(expected_rabbit_task, received_rabbit_task)
        self.assertTrue(same, error_message(expected_rabbit_task, received_rabbit_task))


import json
import copy

path = 'resources/default_values.json'
all_default_values = json.load(open(path))


class TestcaseData(object):
    def __init__(self, testcasename):

        self.testcasename = testcasename

        # Will be filled by user:
        self.dataset = DatasetData()
        self.file1 = FileData()
        self.file2 = FileData()

        # Message identifier
        # Together with the test case identifier, these will
        # be used to compare with the correct outcomes!
        self.dataset.testmessage_identifier = 'dataset1'
        self.file1.testmessage_identifier = 'file1'
        self.file2.testmessage_identifier = 'file2'

        # Taken from default, if not set:
        self.prefix     = str(all_default_values['defaults_connector']['prefix'])
        self.data_node  = str(all_default_values['defaults_connector']['data_node'])
        self.prefix     = str(all_default_values['defaults_connector']['prefix'])
        self.solr_url   = str(all_default_values['defaults_connector']['solr_url'])
        self.thredds_service_path  = str(all_default_values['defaults_connector']['thredds_service_path'])
        self.url_messaging_service = str(all_default_values['defaults_connector']['url_messaging_service'])
        self.messaging_exchange    = str(all_default_values['defaults_connector']['messaging_exchange'])
        self.rabbit_username       = str(all_default_values['defaults_connector']['rabbit_username'])
        self.rabbit_password       = str(all_default_values['defaults_connector']['rabbit_password'])

    def connector_args(self):
        args = dict(
            data_node=self.data_node,
            prefix=self.prefix,
            solr_url=self.solr_url,
            thredds_service_path=self.thredds_service_path,
            url_messaging_service=self.url_messaging_service,
            messaging_exchange=self.messaging_exchange,
            rabbit_username=self.rabbit_username,
            rabbit_password=self.rabbit_password
        )
        return args

    def dataset_args(self):
        self.dataset.testcase_identifier = self.testcasename
        return self.dataset._args()

    def file1_args(self):
        self.file1.testcase_identifier = self.testcasename
        return self.file1._args()

    def file2_args(self):
        self.file2.testcase_identifier = self.testcasename
        return self.file2._args()

    def prettyprint(self, short=True):
        prettystring = '*'*30
        prettystring += '***' + self.description
        prettystring = '*'*30
        if short:
            pass
        else:
            pass
        return prettystring


class DatasetData(object):
    def __init__(self, comment=None):

        # From parent:
        self.testcase_identifier = 'foo'
        self.testmessage_identifier = None

        # Can be set:
        self.comment = comment

        # Has to be set:
        self.drs_id = None
        self.version_number = None

    def _args(self):
        args = dict(
            drs_id=self.drs_id,
            version_number=self.version_number,
            testcase_identifier=self.testcase_identifier,
            testmessage_identifier=self.testmessage_identifier
        )
        if self.comment is not None:
            args['comment'] = self.comment
        return args


class FileData(object):
    def __init__(self, comment=None):

        # From parent:
        self.testcase_identifier = 'bar'
        self.testmessage_identifier = None

        # Can be set:
        self.comment = comment

        # Has to be set:
        self.filename = None
        self.handle = None

        # Taken from default, if not set:
        self.file_version  = str(all_default_values['defaults_files']['file_version'])
        self.checksum      = str(all_default_values['defaults_files']['checksum'])
        self.checksum_type = str(all_default_values['defaults_files']['checksum_type'])
        self.file_size     = str(all_default_values['defaults_files']['file_size'])
        self.publish_path  = str(all_default_values['defaults_files']['publish_path'])

    def get_entire_handle(self):
        handle = str(all_default_values['defaults_connector']['prefix'])+'/esgf_testfile_'+str(self.filename)+'_'+str(self.handle)
        if self.file_version is not None:
            handle += str(self.file_version)
        return handle

    def _args(self):
        if self.filename is None:
            raise ValueError("Filename missing!")
        if self.handle is None:
            raise ValueError("Handle missing!")
        if self.testmessage_identifier is None:
            raise ValueError("Test Message Identifier missing!")

        args = dict(
            file_name=self.filename,
            file_handle=self.get_entire_handle(),
            checksum=self.checksum,
            checksum_type=self.checksum_type,
            file_size=self.file_size,
            publish_path=self.publish_path+'/'+self.filename,
            testcase_identifier=self.testcase_identifier,
            testmessage_identifier=self.testmessage_identifier
        )

        if self.file_version is not None:
            args['file_version'] = self.file_version
        if self.comment is not None:
            args['comment'] = self.comment
        return args




import unittest
import mock
import logging
import json
import requests
import esgfpid.solr.solr
import tests.mocks.responsemock


# Logging:
LOGGER = logging.getLogger(__name__)
LOGGER.addHandler(logging.NullHandler())

# Test resources:
from tests.resources.IGNORE.TESTVALUES_IGNORE import TESTVALUES_SOLR
SOLR_URL = 'http://'+TESTVALUES_SOLR['solr_url']
PREFIX = TESTVALUES_SOLR['prefix']
DRS_ID = TESTVALUES_SOLR['drs_id']
VERSION_NUMBER =  TESTVALUES_SOLR['version_number']
DATA_NODE = TESTVALUES_SOLR['data_node']
FILES = TESTVALUES_SOLR['files']

class SolrTask1IntegrationTestCase(unittest.TestCase):

    def setUp(self):
        LOGGER.info('######## Next test ##########')

    def tearDown(self):
        LOGGER.info('#############################')

    def make_testsolr_with_access(self):
        testsolr = esgfpid.solr.solr.SolrInteractor(
            solr_url = SOLR_URL,
            prefix = PREFIX
        )
        return testsolr

    # Actual tests:

    def test_retrieve_file_handles_of_same_dataset(self):

        # Test variables
        testsolr = self.make_testsolr_with_access()
        drs_id = DRS_ID
        version_number = VERSION_NUMBER
        data_node = DATA_NODE

        # Run code to be tested:
        resp = testsolr.retrieve_file_handles_of_same_dataset(
            drs_id=drs_id,
            version_number=version_number,
            data_node=data_node
        )

        # Check result:
        expected = FILES
        ok = (set(resp) == set(expected))
        self.assertTrue(ok, 'Solr returned:\n'+str(resp)+'\nExpected:\n'+str(expected))

    def test_retrieve_file_handles_of_same_dataset_other_node(self):

        # Test variables
        testsolr = self.make_testsolr_with_access()
        drs_id = DRS_ID
        version_number = VERSION_NUMBER
        data_node = 'bogus.de'

        # Run code to be tested:
        resp = testsolr.retrieve_file_handles_of_same_dataset(
            drs_id=drs_id,
            version_number=version_number,
            data_node=data_node
        )

        # Check result:
        expected = FILES
        ok = (set(resp) == set(expected))
        self.assertTrue(ok, 'Solr returned:\n'+str(resp)+'\nExpected:\n'+str(expected))

    def test_retrieve_file_handles_of_same_dataset_empty_lists(self):

        # Test variables
        testsolr = self.make_testsolr_with_access()
        drs_id = DRS_ID
        version_number = '9999999' # hopefully does not exist, I want an empty list back
        data_node = 'bogus.de'

        # Run code to be tested:
        resp = testsolr.retrieve_file_handles_of_same_dataset(
            drs_id=drs_id,
            version_number=version_number,
            data_node=data_node
        )

        # Check result:
        expected = []
        ok = (set(resp) == set(expected))
        self.assertTrue(ok, 'Solr returned:\n'+str(resp)+'\nExpected:\n'+str(expected))

    def test_retrieve_file_handles_of_same_dataset_errors(self):

        # Test variables
        testsolr = esgfpid.solr.solr.SolrInteractor(
            solr_url = 'https://ihopethisdoesnotexist.de',
            prefix = 'foo'
        )
        drs_id = 'foo'
        version_number = '9999999'
        data_node = 'bogus.de'

        # Run code to be tested and check exception:
        with self.assertRaises(esgfpid.exceptions.SolrError) as raised:
            resp = testsolr.retrieve_file_handles_of_same_dataset(
                drs_id=drs_id,
                version_number=version_number,
                data_node=data_node
            )
        self.assertIn('Failure in both queries.', raised.exception.message)


import unittest
import mock
import logging
import json
import sys
sys.path.append("..")
import esgfpid.rabbit

LOGGER = logging.getLogger(__name__)
LOGGER.addHandler(logging.NullHandler())

class RabbitTestCase(unittest.TestCase):

    def setUp(self):
        LOGGER.info('######## Next test ##########')
        self.testrabbit = self.__make_testrabbit_syn()

    def tearDown(self):
        LOGGER.info('#############################')

    def __get_args_dict(self):
        return dict(
            urls_fallback='www.rabbit.foo',
            url_preferred=None,
            exchange_name='exch',
            username='rogerRabbit',
            password='mySecretCarrotDream'
        )

    @mock.patch('esgfpid.defaults')
    def __make_testrabbit_syn(self, defaults_patch):

        # Patch defaults to create a synchronous object:
        defaults_patch.RABBIT_ASYNCHRONOUS = False
        reload(esgfpid.rabbit.rabbit)

        # Create object
        args = self.__get_args_dict()
        testrabbit = esgfpid.rabbit.rabbit.RabbitMessageSender(**args)

        # Replace connector with mock:
        testrabbit._RabbitMessageSender__server_connector = mock.MagicMock()

        return testrabbit

    # Tests

    #
    # Synchronous:
    #

    @mock.patch('esgfpid.defaults')
    def test_constructor_mandatory_args_(self, defaults_patch):

        # Preparation:
        defaults_patch.RABBIT_ASYNCHRONOUS = False # <-- This is tested
        reload(esgfpid.rabbit.rabbit)

        # Test variables:
        args = self.__get_args_dict()

        # Run code to be tested:
        testrabbit = esgfpid.rabbit.rabbit.RabbitMessageSender(**args)

        # Check result
        self.assertIsInstance(testrabbit, esgfpid.rabbit.rabbit.RabbitMessageSender)
        serverconn = testrabbit._RabbitMessageSender__server_connector
        self.assertIsInstance(serverconn, esgfpid.rabbit.synchronous.SynchronousServerConnector,
            'Wrong type: %s' % type(serverconn))

    def test_open_rabbit_connection(self):

        # Make test rabbit:
        testrabbit = self.__make_testrabbit_syn()
        
        # Run code to be tested:
        testrabbit.open_rabbit_connection()

        # Check result:
        mock_connector = testrabbit._RabbitMessageSender__server_connector
        mock_connector.open_rabbit_connection.assert_called_with()

    def test_close_rabbit_connection(self):

        # Make test rabbit:
        testrabbit = self.__make_testrabbit_syn()

        # Replace the connector with a mock:
        testrabbit._RabbitMessageSender__server_connector = mock.MagicMock()
        
        # Run code to be tested:
        testrabbit.close_rabbit_connection()

        # Check result:
        mock_connector = testrabbit._RabbitMessageSender__server_connector
        mock_connector.close_rabbit_connection.assert_called_with()

    def test_send_message_to_queue(self):

        # Test variables
        msg = 'FOOBAR'

        # Make test rabbit and eplace the connector with a mock:
        testrabbit = self.__make_testrabbit_syn()
                
        # Run code to be tested
        testrabbit.send_message_to_queue(msg)

        # Check result
        mock_connector = testrabbit._RabbitMessageSender__server_connector
        mock_connector.send_message_to_queue.assert_called_with(msg)


    def test_is_finished(self):

        # Make test rabbit and eplace the connector with a mock:
        testrabbit = self.__make_testrabbit_syn()
                
        # Run code to be tested
        res = testrabbit.is_finished()

        # Check result
        self.assertIsNone(res)



import unittest
import mock
import logging
import json
import requests
import esgfpid.solr.solr

# Logging:
LOGGER = logging.getLogger(__name__)
LOGGER.addHandler(logging.NullHandler())

# Test resources:
from tests.resources.IGNORE.TESTVALUES_IGNORE import TESTVALUES_SOLR
SOLR_URL_HTTP = 'http://'+TESTVALUES_SOLR['solr_url']
SOLR_URL_HTTPS = 'https://'+TESTVALUES_SOLR['solr_url']
PREFIX = TESTVALUES_SOLR['prefix']

class SolrServerIntegrationTestCase(unittest.TestCase):

    def setUp(self):
        LOGGER.info('######## Next test ##########')

    def tearDown(self):
        LOGGER.info('#############################')

    # Actual tests:

    def test_send_query_ok(self):

        # Test variables
        testconn = esgfpid.solr.serverconnector.SolrServerConnector(solr_url = SOLR_URL_HTTP)
        query = dict(
            distrib=False,
            type='File',
            format='application/solr+json',
            limit=0
        )

        # Run code to be tested:
        response = testconn.send_query(query)

        # Check result:
        self.assertTrue('responseHeader' in response, 'Solr response was not correct JSON: '+str(response))

    def test_send_query_ok_warnings_1_are_enabled(self):
        '''
        IMPORTANT:
        This only works if this test is run before the disabled test.
        It seems that the disabling stays...
        That is why there is numbers in the test name - they
        seem to be run in alphabetical order.
        '''

        LOGGER.debug('*** Without disabling ***')

        # Test variables
        testconn = esgfpid.solr.serverconnector.SolrServerConnector(
            solr_url = SOLR_URL_HTTPS,
            disable_insecure_request_warning=False,
            https_verify=False
        )
        query = dict(
            distrib=False,
            type='File',
            format='application/solr+json',
            limit=0
        )

        # Capture stderr:
        import sys
        from StringIO import StringIO
        saved_stdout = sys.stderr
        out = StringIO()
        sys.stderr = out
        # Run code to be tested:
        response = testconn.send_query(query)
        # Retrieve stderr and reset stderr:
        output = out.getvalue().strip()
        sys.stdout = saved_stdout

        # Check result:
        expected_warning = 'InsecureRequestWarning: Unverified HTTPS request is being made'
        self.assertTrue(expected_warning in output, 'Warning missing: '+output)


    def test_send_query_ok_warnings_2_disabled(self):

        LOGGER.debug('*** With disabling ***')

        # Test variables
        testconn = esgfpid.solr.serverconnector.SolrServerConnector(
            solr_url = SOLR_URL_HTTPS,
            disable_insecure_request_warning=True,
            https_verify=False
        )
        query = dict(
            distrib=False,
            type='File',
            format='application/solr+json',
            limit=0
        )

        # Capture stderr:
        import sys
        from StringIO import StringIO
        saved_stdout = sys.stderr
        out = StringIO()
        sys.stderr = out
        # Run code to be tested:
        response = testconn.send_query(query)
        # Retrieve stderr and reset stderr:
        output = out.getvalue().strip()
        sys.stdout = saved_stdout

        # Check result:
        expected_warning = 'InsecureRequestWarning: Unverified HTTPS request is being made'
        self.assertFalse(expected_warning in output, 'Some stderr occurred, but we tried switching it off: '+output)



import json
import mock

class MockSolrInteractor(object):

    def __init__(self, solr_url='mock_solr_url', prefix='mock_prefix', https_verify=True, previous_files=None, raise_error=False):
        self.solr_url = solr_url
        self.prefix = prefix
        self.https_verify = https_verify
        self.previous_files = previous_files
        if previous_files is None:
            self.previous_files = []
        if previous_files == 'NONE':
            self.previous_files = None
        self.datasethandles_or_versionnumbers_of_allversions = dict(dataset_handles=None, version_numbers=None)
        self.raise_error = raise_error

    def retrieve_file_handles_of_same_dataset(self, *args, **kwargs):
        if self.raise_error != False:
            raise self.raise_error
        else:
            return self.previous_files

    def has_access(self):
        return True

    def retrieve_datasethandles_or_versionnumbers_of_allversions(self, drs_id):
        if self.raise_error != False:
            raise self.raise_error
        else:
            return self.datasethandles_or_versionnumbers_of_allversions

    def is_switched_off(self):
        return False


import unittest
import mock
import logging
import copy
import os
import esgfpid.rabbit.asynchronous.thread_shutter
from esgfpid.rabbit.asynchronous.exceptions import OperationNotAllowed

LOGGER = logging.getLogger(__name__)
LOGGER.addHandler(logging.NullHandler())

UNCONFIRMED_TAGS = [1,2,3,4]
UNCONFIRMED_MESSAGES = {'1':'foo1', '2':'foo2', '3':'foo3', '4':'foo4'}

#
# Create mock ups
#

class MockFeeder(object):

    def __init__(self, num_unpublished=0, reduce_at_every_call=False):
        self.num = num_unpublished
        self.reduce = reduce_at_every_call

    def get_num_unpublished(self):
        num = self.num
        if num < 0:
            num = 0
        if self.reduce:
            self.num -= 1
        return num

class MockThread(object):
    def __init__(self):
        self._connection = mock.MagicMock()
        self.ERROR_CODE_CONNECTION_CLOSED_BY_USER = 999
        self.ERROR_TEXT_CONNECTION_FORCE_CLOSED= 'foo'
        self.ERROR_TEXT_CONNECTION_NORMAL_SHUTDOWN = 'bar'
    def _make_permanently_closed_by_user(self):
        pass

class ThreadShutterTestCase(unittest.TestCase):

    def setUp(self):
        LOGGER.info('######## Next test ########## (%s)' % os.path.basename(__file__))

    def tearDown(self):
        LOGGER.info('#############################')

    def make_shutter(self, feeder=None):

        # Mocked:
        self.thread = MockThread()
        self.feeder = feeder or MockFeeder()
        self.statemachine = esgfpid.rabbit.asynchronous.thread_statemachine.StateMachine()
        self.confirmer = esgfpid.rabbit.asynchronous.thread_confirmer.ConfirmReactor()
        shutter = esgfpid.rabbit.asynchronous.thread_shutter.ShutDowner(
            self.thread,
            self.statemachine,
            self.confirmer,
            self.feeder)
        self.statemachine._StateMachine__state = self.statemachine._StateMachine__IS_AVAILABLE
        return shutter

    def add_leftovers_to_confirmer(self):
        self.confirmer._ConfirmReactor__unconfirmed_delivery_tags = copy.copy(UNCONFIRMED_TAGS)
        self.confirmer._ConfirmReactor__unconfirmed_messages_dict = copy.copy(UNCONFIRMED_MESSAGES)

    # Tests

    #
    # Force finish
    #

    def test_force_finish_no_leftovers_ok(self):

        # Preparation:
        shutter = self.make_shutter()

        # Run code to be tested:
        shutter.force_finish()

        # Check result state:
        state = self.statemachine._StateMachine__state
        exp = self.statemachine._StateMachine__PERMANENTLY_UNAVAILABLE
        self.assertEquals(state, exp)
        # Check leftovers:
        self.assertEquals(self.confirmer.get_num_unconfirmed(), 0)
        self.assertEquals(self.feeder.get_num_unpublished(), 0)

    def test_force_finish_leftovers_ok(self):

        # Preparation:
        shutter = self.make_shutter()
        self.add_leftovers_to_confirmer()

        # Run code to be tested:
        shutter.force_finish()

        # Check result state:
        state = self.statemachine._StateMachine__state
        exp = self.statemachine._StateMachine__PERMANENTLY_UNAVAILABLE
        self.assertEquals(state, exp)
        # Check leftovers:
        self.assertEquals(self.confirmer.get_num_unconfirmed(), 4)
        self.assertEquals(self.feeder.get_num_unpublished(), 0)

    #
    # Gentle finish
    #

    def test_gently_finish_no_leftovers_ok(self):

        # Preparation:
        shutter = self.make_shutter()

        # Run code to be tested:
        shutter.finish_gently()

        # Check result state:
        state = self.statemachine._StateMachine__state
        exp = self.statemachine._StateMachine__PERMANENTLY_UNAVAILABLE
        self.assertEquals(state, exp)
        # Check leftovers:
        self.assertEquals(self.confirmer.get_num_unconfirmed(), 0)
        self.assertEquals(self.feeder.get_num_unpublished(), 0)

    def test_gently_finish_with_unpublished_messages_ok(self):

        # Preparation:
        shutter = self.make_shutter(MockFeeder(3, True))

        # Run code to be tested:
        shutter.finish_gently()

        # Check result state:
        state = self.statemachine._StateMachine__state
        exp = self.statemachine._StateMachine__PERMANENTLY_UNAVAILABLE
        self.assertEquals(state, exp)
        # Check leftovers:
        self.assertEquals(self.confirmer.get_num_unconfirmed(), 0)
        self.assertEquals(self.feeder.get_num_unpublished(), 0)

    def test_gently_finish_with_unconfirmed_messages_ok(self):
        print("\nThis test takes time, as it calls the finish_gently method while not all messages are done.")

        # Preparation:
        shutter = self.make_shutter()
        self.add_leftovers_to_confirmer()

        # Run code to be tested:
        shutter.finish_gently()

        # Check result state:
        state = self.statemachine._StateMachine__state
        exp = self.statemachine._StateMachine__PERMANENTLY_UNAVAILABLE
        self.assertEquals(state, exp)
        # Check leftovers:
        self.assertEquals(self.confirmer.get_num_unconfirmed(), 4)
        self.assertEquals(self.feeder.get_num_unpublished(), 0)

    def test_gently_finish_with_leftovers_not_progressing_1(self):

        # Preparation:
        shutter = self.make_shutter(MockFeeder(4))
        self.add_leftovers_to_confirmer()
        self.statemachine._StateMachine__state = self.statemachine._StateMachine__PERMANENTLY_UNAVAILABLE

        # Run code to be tested:
        shutter.finish_gently()

        # Check result state:
        state = self.statemachine._StateMachine__state
        exp = self.statemachine._StateMachine__PERMANENTLY_UNAVAILABLE
        self.assertEquals(state, exp)
        # Check leftovers:
        self.assertEquals(self.confirmer.get_num_unconfirmed(), 4)
        self.assertEquals(self.feeder.get_num_unpublished(), 4)

    def test_gently_finish_with_leftovers_not_progressing_2(self):

        # Preparation:
        shutter = self.make_shutter(MockFeeder(4))
        self.statemachine._StateMachine__state = self.statemachine._StateMachine__PERMANENTLY_UNAVAILABLE
        self.statemachine.closed_by_publisher = True

        # Run code to be tested:
        shutter.finish_gently()

        # Check result state:
        state = self.statemachine._StateMachine__state
        exp = self.statemachine._StateMachine__PERMANENTLY_UNAVAILABLE
        self.assertEquals(state, exp)
        # Check leftovers:
        self.assertEquals(self.confirmer.get_num_unconfirmed(), 0)
        self.assertEquals(self.feeder.get_num_unpublished(), 4)

    def test_gently_finish_with_leftovers_not_progressing_3(self):

        # Preparation:
        shutter = self.make_shutter(MockFeeder(4))
        self.statemachine._StateMachine__state = self.statemachine._StateMachine__PERMANENTLY_UNAVAILABLE
        self.statemachine.could_not_connect = True

        # Run code to be tested:
        shutter.finish_gently()

        # Check result state:
        state = self.statemachine._StateMachine__state
        exp = self.statemachine._StateMachine__PERMANENTLY_UNAVAILABLE
        self.assertEquals(state, exp)
        # Check leftovers:
        self.assertEquals(self.confirmer.get_num_unconfirmed(), 0)
        self.assertEquals(self.feeder.get_num_unpublished(), 4)

    def test_gently_finish_with_leftovers_timeout(self):
        print("\nThis test takes time, as it calls the finish_gently method while not all messages are done.")

        # Preparation:
        shutter = self.make_shutter(MockFeeder(4))

        # Run code to be tested:
        shutter.finish_gently()

        # Check result state:
        state = self.statemachine._StateMachine__state
        exp = self.statemachine._StateMachine__PERMANENTLY_UNAVAILABLE
        self.assertEquals(state, exp)
        # Check leftovers:
        self.assertEquals(self.confirmer.get_num_unconfirmed(), 0)
        self.assertEquals(self.feeder.get_num_unpublished(), 4)


import unittest
import mock
import logging
import json
import esgfpid.assistant.shoppingcart
import tests.mocks.rabbitmock
import tests.mocks.solrmock
import tests.utils as utils
from tests.utils import compare_json_return_errormessage as error_message
from esgfpid.defaults import ROUTING_KEY_BASIS as ROUTING_KEY_BASIS


# Logging
LOGGER = logging.getLogger(__name__)
LOGGER.addHandler(logging.NullHandler())

# Test resources:
from resources.TESTVALUES import TESTVALUES as TESTVALUES

class ShoppingCartTestCase(unittest.TestCase):

    def setUp(self):
        LOGGER.info('######## Next test (%s) ##########', __name__)

    def tearDown(self):
        LOGGER.info('#############################')

    def make_test_coupler_for_sending_messages(self):
        testcoupler = esgfpid.coupling.Coupler(
            handle_prefix = TESTVALUES['prefix'],
            messaging_service_urls_open = TESTVALUES['url_rabbit_open'],
            messaging_service_url_trusted = TESTVALUES['url_rabbit_trusted'],
            messaging_service_exchange_name = TESTVALUES['rabbit_exchange_name'],
            solr_url = TESTVALUES['solr_url'],
            messaging_service_username_open = TESTVALUES['rabbit_username_open'],
            messaging_service_username_trusted = TESTVALUES['rabbit_username_trusted'],
            messaging_service_password = TESTVALUES['rabbit_password'],
            solr_switched_off = True,
            solr_https_verify=False
        )
        # Replace objects that interact with servers with mocks
        testcoupler._Coupler__rabbit_message_sender = tests.mocks.rabbitmock.SimpleMockRabbitSender()
        return testcoupler

    def __get_received_message_from_rabbit_mock(self, coupler, index):
        return coupler._Coupler__rabbit_message_sender.received_messages[index]

    ### Actual test cases: ###

    def test_init_ok(self):

        # Test variables
        testcoupler = self.make_test_coupler_for_sending_messages()
        prefix = TESTVALUES['prefix']
  
        # Run code to be tested:
        assistant = esgfpid.assistant.shoppingcart.ShoppingCartAssistant(
            prefix=prefix,
            coupler=testcoupler
        )

        # Check result:
        self.assertIsInstance(assistant, esgfpid.assistant.shoppingcart.ShoppingCartAssistant, 'Constructor fail.')

    def test_add_content_several(self):

        # Test variables
        testcoupler = self.make_test_coupler_for_sending_messages()
        prefix = TESTVALUES['prefix']
  
        # Preparations
        assistant = esgfpid.assistant.shoppingcart.ShoppingCartAssistant(
            prefix=prefix,
            coupler=testcoupler
        )

        # Run code to be tested:
        assistant.make_shopping_cart_pid({'foo':'foo', 'bar':'bar'})

        # Check result:
        expected_rabbit_task = {
            "handle": "hdl:"+prefix+'/339427df-edbd-3f43-acf2-80ddc7729f27',
            "operation": "shopping_cart",
            "message_timestamp":"anydate",
            "data_cart_content":{'foo':'foo', 'bar':'bar'},
            "ROUTING_KEY": ROUTING_KEY_BASIS+'cart.datasets'
        }
        received_rabbit_task = self.__get_received_message_from_rabbit_mock(testcoupler, 0)
        same = utils.is_json_same(expected_rabbit_task, received_rabbit_task)
        self.assertTrue(same, error_message(expected_rabbit_task, received_rabbit_task))

    def test_add_content_some_none(self):

        # Test variablesl
        testcoupler = self.make_test_coupler_for_sending_messages()
        prefix = TESTVALUES['prefix']
  
        # Preparations
        assistant = esgfpid.assistant.shoppingcart.ShoppingCartAssistant(
            prefix=prefix,
            coupler=testcoupler
        )

        # Run code to be tested:
        assistant.make_shopping_cart_pid({'foo':'foo', 'bar':None})

        # Check result:
        expected_rabbit_task = {
            "handle": "hdl:"+prefix+'/339427df-edbd-3f43-acf2-80ddc7729f27',
            "operation": "shopping_cart",
            "message_timestamp":"anydate",
            "data_cart_content":{'foo':'foo', 'bar':None},
            "ROUTING_KEY": ROUTING_KEY_BASIS+'cart.datasets'
        }
        received_rabbit_task = self.__get_received_message_from_rabbit_mock(testcoupler, 0)
        same = utils.is_json_same(expected_rabbit_task, received_rabbit_task)
        self.assertTrue(same, error_message(expected_rabbit_task, received_rabbit_task))

    def test_add_content_several_old(self):

        # Test variables
        testcoupler = self.make_test_coupler_for_sending_messages()
        prefix = TESTVALUES['prefix']
        content1 = {'foo':'foo', 'bar':'bar'}
        content2 = {'foo':'foo', 'bar': None}
        content3 = {'foo':'foo', 'bar':'hdl:bar'}


        # Preparations
        assistant = esgfpid.assistant.shoppingcart.ShoppingCartAssistant(
            prefix=prefix,
            coupler=testcoupler
        )

        # Run code to be tested:
        pid1 = assistant.make_shopping_cart_pid(content1)
        received_rabbit_task1 = self.__get_received_message_from_rabbit_mock(testcoupler, 0)
        pid2 = assistant.make_shopping_cart_pid(content2)
        received_rabbit_task2 = self.__get_received_message_from_rabbit_mock(testcoupler, 1)
        pid3 = assistant.make_shopping_cart_pid(content3)
        received_rabbit_task3 = self.__get_received_message_from_rabbit_mock(testcoupler, 2)

        # Check result:
        expected_handle_both_cases = "hdl:"+prefix+"/339427df-edbd-3f43-acf2-80ddc7729f27"
        expected_rabbit_task1 = {
            "handle": expected_handle_both_cases,
            "operation": "shopping_cart",
            "message_timestamp":"anydate",
            "data_cart_content":content1,
            "ROUTING_KEY": ROUTING_KEY_BASIS+'cart.datasets'
        }
        expected_rabbit_task2 = {
            "handle": expected_handle_both_cases,
            "operation": "shopping_cart",
            "message_timestamp":"anydate",
            "data_cart_content":content2,
            "ROUTING_KEY": ROUTING_KEY_BASIS+'cart.datasets'
        }
        expected_rabbit_task3 = {
            "handle": expected_handle_both_cases,
            "operation": "shopping_cart",
            "message_timestamp":"anydate",
            "data_cart_content":content3,
            "ROUTING_KEY": ROUTING_KEY_BASIS+'cart.datasets'
        }
        same1 = utils.is_json_same(expected_rabbit_task1, received_rabbit_task1)
        same2 = utils.is_json_same(expected_rabbit_task2, received_rabbit_task2)
        same3 = utils.is_json_same(expected_rabbit_task3, received_rabbit_task3)
        self.assertTrue(same1, error_message(expected_rabbit_task1, received_rabbit_task1))
        self.assertTrue(same2, error_message(expected_rabbit_task2, received_rabbit_task2))
        self.assertTrue(same3, error_message(expected_rabbit_task3, received_rabbit_task3))
        self.assertTrue(pid1==pid2, 'Pids 1&2 are not the same.')
        self.assertTrue(pid1==pid3, 'Pids 1&3 are not the same.')



import unittest
import mock
import logging
import json
import pika
import esgfpid.rabbit.rabbit
import tests.mocks.rabbitmock
import tests.mocks.solrmock
import tests.json_utils

# Logging
LOGGER = logging.getLogger(__name__)
LOGGER.addHandler(logging.NullHandler())

# Test resources:
from tests.resources.IGNORE.TESTVALUES_IGNORE import TESTVALUES_REAL

class RabbitIntegrationTestCase(unittest.TestCase):

    def make_rabbit_with_access(self):
        testrabbit = esgfpid.rabbit.rabbit.RabbitMessageSender(
            exchange_name=TESTVALUES_REAL['exchange_name'],
            urls_fallback=TESTVALUES_REAL['url_messaging_service'],
            url_preferred=None,
            username=TESTVALUES_REAL['rabbit_username'],
            password=TESTVALUES_REAL['rabbit_password'],
        )
        return testrabbit

    # Tests

    def test_constructor_mandatory_args(self):
        testrabbit = self.make_rabbit_with_access()
        self.assertIsInstance(testrabbit, esgfpid.rabbit.rabbit.RabbitMessageSender)

    def test_constructor_wrong_credentials(self):
        testrabbit = esgfpid.rabbit.rabbit.RabbitMessageSender(
            exchange_name=TESTVALUES_REAL['exchange_name'],
            urls_fallback=TESTVALUES_REAL['url_messaging_service'],
            url_preferred=None,
            username='wrong',
            password='wrong',
        )
        with self.assertRaises(pika.exceptions.ProbableAuthenticationError):
            testrabbit.open_rabbit_connection()


    def test_send_message_to_queue_ok(self):

        # Test variables:
        testrabbit = self.make_rabbit_with_access()
        routing_key = TESTVALUES_REAL['routing_key']
        message_body = '{"stuff":"foo"}'

        # Run code to be tested:
        delivered = testrabbit.send_message_to_queue(routing_key=routing_key, message=message_body)

        # Check result:
        self.assertTrue(delivered, 'Message not delivered!')

    def test_send_message_to_queue_wrong_routing_key(self):

        # Test variables:
        testrabbit = self.make_rabbit_with_access()
        routing_key = 'random'
        message_body = '{"stuff":"foo"}'

        # Run code to be tested:
        delivered = testrabbit.send_message_to_queue(routing_key=routing_key, message=message_body)

        # Check result:
        self.assertFalse(delivered, 'Message was delivered, but should not!')

    def test_send_message_to_queue_no_queue(self):

        # Test variables:
        routing_key = TESTVALUES_REAL['routing_key']
        message_body = '{"stuff":"foo"}'
        testrabbit = esgfpid.rabbit.rabbit.RabbitMessageSender(
            exchange_name=TESTVALUES_REAL['exchange_no_queue'],
            urls_fallback=TESTVALUES_REAL['url_messaging_service'],
            url_preferred=None,
            username=TESTVALUES_REAL['rabbit_username'],
            password=TESTVALUES_REAL['rabbit_password'],
        )

        # Run code to be tested:
        delivered = testrabbit.send_message_to_queue(routing_key=routing_key, message=message_body)

        # Check result:
        self.assertFalse(delivered, 'Message was delivered, but should not!')



import unittest
import mock
import logging
import json
import requests
import esgfpid.solr.solr
import tests.mocks.responsemock


# Logging:
LOGGER = logging.getLogger(__name__)
LOGGER.addHandler(logging.NullHandler())

# Load some data that is needed for testing
PATH_RES = tests.utils.get_neighbour_directory(__file__, 'resources')
SOLR_RESPONSE = json.load(open(PATH_RES+'/solr_response.json'))
from tests.resources.TESTVALUES import TESTVALUES as TESTVALUES


class SolrServerConnectorTestCase(unittest.TestCase):

    def make_testsolr_with_access(self):
        testsolr = esgfpid.solr.serverconnector.SolrServerConnector(
            solr_url = TESTVALUES['solr_url']
        )
        return testsolr


    # Actual tests:

    def test_init_ok(self):

        # Run code to be tested:
        testsolr = esgfpid.solr.serverconnector.SolrServerConnector(
            solr_url = 'blablabla///'
        )

        # Check result
        self.assertIsInstance(testsolr, esgfpid.solr.serverconnector.SolrServerConnector, 'Constructor fail.')
        self.assertFalse(testsolr._SolrServerConnector__https_verify, 'HTTPS verify not set to False.')
        self.assertEquals(testsolr._SolrServerConnector__solr_url, 'blablabla', 'Wrong url.')


    def test_init_with_optional_args(self):

        # Run code to be tested:
        testsolr = esgfpid.solr.serverconnector.SolrServerConnector(
            solr_url = 'blablabla',
            https_verify = True,
            disable_insecure_request_warning = True
        )

        self.assertIsInstance(testsolr, esgfpid.solr.serverconnector.SolrServerConnector, 'Constructor fail.')
        self.assertTrue(testsolr._SolrServerConnector__https_verify, 'HTTPS verify not set to True.')

    @mock.patch('esgfpid.solr.serverconnector.requests.get')
    def test_send_query_ok_patched(self, getpatch):

        # Define the replacement for the patched method:
        mock_response = tests.mocks.responsemock.MockSolrResponse(success=True)
        getpatch.return_value = mock_response

        # Test variables
        testsolr = self.make_testsolr_with_access()

        # Run code to be tested:
        response = testsolr.send_query('blah')

        # Check result:
        # We expect the mock response to be returned and parsed to JSON.
        self.assertTrue('responseHeader' in response, 'Solr response was not correct JSON: '+str(response))
        self.assertTrue(len(response['facet_counts']['facet_fields']['bla'])==4, 'JSON content was not transmitted correctly.')

    @mock.patch('esgfpid.solr.serverconnector.requests.get')
    def test_send_query_200_empty_patched(self, getpatch):

        # Define the replacement for the patched method:
        mock_response = tests.mocks.responsemock.MockSolrResponse(success=True, content='NONE')
        getpatch.return_value = mock_response

        # Test variables
        testsolr = self.make_testsolr_with_access()

        # Run code to be tested and check exception:
        with self.assertRaises(esgfpid.exceptions.SolrError) as raised:
            response = testsolr.send_query('blah')
        self.assertIn('empty response', raised.exception.message)

    @mock.patch('esgfpid.solr.serverconnector.requests.get')
    def test_send_query_broken_json_patched(self, getpatch):

        # Define the replacement for the patched method:
        broken_json = '{"responseHeader":{}, "response":{}, "facet_counts": {}'
        mock_response = tests.mocks.responsemock.MockSolrResponse(success=True, content=broken_json)
        getpatch.return_value = mock_response

        # Test variables
        testsolr = self.make_testsolr_with_access()

        # Run code to be tested and check exception:
        with self.assertRaises(esgfpid.exceptions.SolrError) as raised:
            response = testsolr.send_query('blah')
        self.assertIn('no valid JSON', raised.exception.message)

    @mock.patch('esgfpid.solr.serverconnector.requests.get')
    def test_send_query_return_none_patched(self, getpatch):

        # Define the replacement for the patched method:
        getpatch.return_value = None

        # Test variables
        testsolr = self.make_testsolr_with_access()

        # Run code to be tested and check exception:
        with self.assertRaises(esgfpid.exceptions.SolrError) as raised:
            response = testsolr.send_query('blah')
        self.assertIn('no response', raised.exception.message)

    @mock.patch('esgfpid.solr.serverconnector.requests.get')
    def test_send_query_http_404_patched(self, getpatch):

        # Define the replacement for the patched method:
        mock_response = tests.mocks.responsemock.MockSolrResponse(notfound=True)
        getpatch.return_value = mock_response

        # Test variables
        testsolr = self.make_testsolr_with_access()

        # Run code to be tested and check exception:
        with self.assertRaises(esgfpid.exceptions.SolrError) as raised:
            response = testsolr.send_query('blah')
        self.assertIn('HTTP 404', raised.exception.message)

    @mock.patch('esgfpid.solr.serverconnector.requests.get')
    def test_send_query_patched_connection_error(self, getpatch):

        # Define the replacement for the patched method:
        getpatch.side_effect = requests.exceptions.ConnectionError()

        # Test variables
        testsolr = self.make_testsolr_with_access()

        # Run code to be tested and check exception:
        with self.assertRaises(esgfpid.exceptions.SolrError) as raised:
            response = testsolr.send_query('blah')
        self.assertIn('ConnectionError', raised.exception.message)

    @mock.patch('esgfpid.solr.serverconnector.requests.get')
    def test_send_query_patched_5000(self, getpatch):

        # Define the replacement for the patched method:
        mock_response = tests.mocks.responsemock.MockSolrResponse(status_code=5000)
        getpatch.return_value = mock_response

        # Test variables
        testsolr = self.make_testsolr_with_access()

        # Run code to be tested and check exception:
        with self.assertRaises(esgfpid.exceptions.SolrError) as raised:
            response = testsolr.send_query('blah')
        self.assertIn('code 5000', raised.exception.message)



import unittest
import mock
import logging
import copy
import os
import esgfpid.rabbit.asynchronous.thread_acceptor
from esgfpid.rabbit.asynchronous.exceptions import OperationNotAllowed

LOGGER = logging.getLogger(__name__)
LOGGER.addHandler(logging.NullHandler())


class ThreadAcceptorTestCase(unittest.TestCase):

    def setUp(self):
        LOGGER.info('######## Next test ########## (%s)' % os.path.basename(__file__))

    def tearDown(self):
        LOGGER.info('#############################')

    def make_acceptor(self):
        self.thread = mock.MagicMock()
        self.feeder = mock.MagicMock()
        self.statemachine = esgfpid.rabbit.asynchronous.thread_statemachine.StateMachine()
        acceptor = esgfpid.rabbit.asynchronous.thread_acceptor.PublicationReceiver(
            self.thread,
            self.statemachine,
            self.feeder)
        self.statemachine._StateMachine__state = self.statemachine._StateMachine__IS_AVAILABLE
        self.thread._connection = mock.MagicMock()
        return acceptor

    # Tests

    #
    # Sending messages
    #

    def test_send_message_ok(self):

        # Preparation:
        acceptor = self.make_acceptor()

        # Run code to be tested:
        acceptor.send_a_message('foo')

        # Check result:
        self.feeder.put_message_into_queue_of_unsent_messages.assert_called_with('foo')
        self.thread._connection.add_timeout.assert_called_with(0, self.feeder.publish_message)


    def test_send_many_messages_ok(self):

        # Preparation:
        acceptor = self.make_acceptor()

        # Run code to be tested:
        acceptor.send_many_messages(['foo','bar'])

        # Check result:
        self.feeder.put_message_into_queue_of_unsent_messages.assert_any_call('bar')
        self.feeder.put_message_into_queue_of_unsent_messages.assert_any_call('foo')
        self.thread._connection.add_timeout.call_count = 2

    def test_resend_message_ok(self):

        # Preparation:
        acceptor = self.make_acceptor()
        channel = mock.MagicMock()
        self.thread._connection._channel = channel
        message = '{"foo":"bar","ROUTING_KEY":"myfoo"}'

        # Run code to be tested:

        acceptor.on_message_not_accepted(channel, 'serverreturnfoo', 'propsfoo', message)

        # Check result:
        exp = '{"original_routing_key": "myfoo", "foo": "bar", "ROUTING_KEY": "cmip6.publisher.HASH.emergency"}'
        self.feeder.put_message_into_queue_of_unsent_messages.assert_called_with(exp)
        self.thread._connection.add_timeout.assert_called_with(0, self.feeder.publish_message)

    def test_send_message_no_more_accept(self):

        # Preparation:
        acceptor = self.make_acceptor()
        self.statemachine._StateMachine__state = self.statemachine._StateMachine__IS_AVAILABLE_BUT_WANTS_TO_STOP

        # Run code to be tested:
        with self.assertRaises(OperationNotAllowed) as e:
            acceptor.send_a_message('foo')
        self.assertIn('Accepting no more messages', e.exception.message)
        with self.assertRaises(OperationNotAllowed) as e:
            acceptor.send_many_messages(['foo','bar'])
        self.assertIn('Accepting no more messages', e.exception.message)

    def test_send_message_cannot_trigger_1(self):

        # Preparation:
        acceptor = self.make_acceptor()
        self.statemachine._StateMachine__state = self.statemachine._StateMachine__WAITING_TO_BE_AVAILABLE
        
        # Run code to be tested:
        acceptor.send_a_message('foo')

        # Check result:
        self.feeder.put_message_into_queue_of_unsent_messages.assert_called_with('foo')
        self.thread._connection.add_timeout.assert_not_called()

    def test_send_message_cannot_trigger_2(self):

        # Preparation:
        acceptor = self.make_acceptor()
        self.statemachine._StateMachine__state = self.statemachine._StateMachine__NOT_STARTED_YET

        # Run code to be tested:
        acceptor.send_a_message('foo')

        # Check result:
        self.feeder.put_message_into_queue_of_unsent_messages.assert_called_with('foo')
        self.thread._connection.add_timeout.assert_not_called()

    def test_send_message_cannot_trigger_3(self):

        # Preparation:
        acceptor = self.make_acceptor()
        self.statemachine._StateMachine__state = self.statemachine._StateMachine__PERMANENTLY_UNAVAILABLE
        self.statemachine.could_not_connect = True

        # Run code to be tested:
        acceptor.send_a_message('foo')

        # Check result:
        self.feeder.put_message_into_queue_of_unsent_messages.assert_called_with('foo')
        self.thread._connection.add_timeout.assert_not_called()

    def test_send_message_cannot_trigger_4(self):

        # Preparation:
        acceptor = self.make_acceptor()
        self.statemachine._StateMachine__state = self.statemachine._StateMachine__PERMANENTLY_UNAVAILABLE
        self.statemachine.could_not_connect = False
        self.statemachine.closed_by_publisher = True

        # Run code to be tested:
        acceptor.send_many_messages(['foo','bar'])

        # Check result:
        self.feeder.put_message_into_queue_of_unsent_messages.assert_any_call('bar')
        self.feeder.put_message_into_queue_of_unsent_messages.assert_any_call('foo')
        self.thread._connection.add_timeout.assert_not_called()



import pika
import logging
from esgfpid.utils import check_presence_of_mandatory_args
from esgfpid.utils import add_missing_optional_args_with_value_none
import esgfpid.defaults
import esgfpid.rabbit.rabbitutils
import esgfpid.utils as utils
import esgfpid.exceptions

LOGGER = logging.getLogger(__name__)
LOGGER.addHandler(logging.NullHandler())

def check_pid_queue_availability(**args):
    rabbit_checker = RabbitChecker(**args)
    return rabbit_checker.check_and_inform()

class RabbitChecker(object):

    #
    # Init
    #

    def __init__(self, **args):
        mandatory_args = ['messaging_service_username', 'messaging_service_password']
        optional_args = ['messaging_service_url_preferred', 'messaging_service_urls', 'print_to_console', 'messaging_service_exchange_name', 'print_success_to_console']
        check_presence_of_mandatory_args(args, mandatory_args)
        add_missing_optional_args_with_value_none(args, optional_args)
        self.__check_if_any_url_specified(args)
        self.__rename_url_args(args)
        self.__adapt_url_args(args)
        self.__define_all_attributes()
        self.__fill_all_attributes(args)
        self.connection = None
        self.channel = None
        self.channel_ok = False

    def __define_all_attributes(self):
        self.__print_errors_to_console = False
        self.__print_success_to_console = False
        self.__default_log_level = logging.DEBUG
        self.__error_messages = []
        self.__rabbit_username = None
        self.__rabbit_password = None
        self.__rabbit_hosts = None # all passed hosts that are NOT the current one!
        self.__current_rabbit_host = None # the only host OR the preferred OR a randomly chosen
        self.__exchange_name = None

    def __check_if_any_url_specified(self, args):
        preferred_given = args['messaging_service_url_preferred'] is not None
        other1 = args['messaging_service_urls'] is not None
        other2 = len(args['messaging_service_urls']) > 0 if other1 else False
        other_given = (other1 and other2)
        if not (preferred_given or other_given):
            raise esgfpid.exceptions.ArgumentError('At least one messaging service URL has to be specified.') 

    def __rename_url_args(self, args):
        args['urls_fallback'] = args['messaging_service_urls']
        del args['messaging_service_urls']
        args['url_preferred'] = args['messaging_service_url_preferred']
        del args['messaging_service_url_preferred']

    def __adapt_url_args(self, args):
        esgfpid.rabbit.rabbitutils.ensure_urls_are_a_list(args, LOGGER)
        esgfpid.rabbit.rabbitutils.set_preferred_url(args, LOGGER)

    def __fill_all_attributes(self, args):
        self.__rabbit_username = args['messaging_service_username']
        self.__rabbit_password = args['messaging_service_password']
        self.__rabbit_hosts = args['urls_fallback']
        self.__current_rabbit_host = args['url_preferred']
        if args['print_to_console'] is not None and args['print_to_console'] == True:
            self.__print_errors_to_console = True
        if args['print_success_to_console'] is not None and args['print_success_to_console'] == True:
            self.__print_success_to_console = True
        if args['messaging_service_exchange_name'] is not None:
            self.__exchange_name = args['messaging_service_exchange_name']


    #
    # Perform the checks
    #

    def check_and_inform(self):
        self.__loginfo('Checking config for PID module (rabbit messaging queue) ...')
        success = self.__iterate_over_all_hosts()
        if success:
            self.__loginfo('Config for PID module (rabbit messaging queue).. ok.')
            self.__loginfo('Successful connection to PID messaging queue at "%s".' % self.__current_rabbit_host)
            self.__define_fallback_exchange() # remove!
        else:
            self.__loginfo('Config for PID module (rabbit messaging queue) .. FAILED!')
            self.__assemble_and_print_error_message()

            if self.channel_ok:
                self.__define_fallback_exchange()

        if self.connection is not None:
            self.connection.close()
        return success

    def __iterate_over_all_hosts(self):
        success = False
        print_conn = True
        print_chan = True
        self.channel_ok = False

        while True:
            try:
                if print_conn:
                    self.__loginfo(' .. checking authentication and connection ...')
                    print_conn = False
                    print_chan = True
                
                self.connection = self.__check_making_rabbit_connection()

                if print_chan:
                    self.__loginfo(' .. checking authentication and connection ... ok.')
                    self.__loginfo(' .. checking channel ...')
                    print_chan = False
                    print_conn = True

                self.channel_ok = False
                self.channel = self.__check_opening_channel(self.connection)
                self.channel_ok = True
                self.__check_exchange_existence(self.channel)

                success = True

                break # success, leave loop

            except ValueError as e:

                if self.__is_url_left(): # stay in loop, try next host
                    utils.logtrace(LOGGER, 'Left URLs: %s', self.__rabbit_hosts)
                    self.__set_next_url()
                    utils.logtrace(LOGGER, 'Now trying: %s', self.__current_rabbit_host)

                else: # definitive fail, leave loop
                    break

        return success
            
    def __is_url_left(self):
        if len(self.__rabbit_hosts) > 0:
            return True
        return False

    def __set_next_url(self):
        self.__current_rabbit_host = self.__rabbit_hosts.pop()

    #
    # Building connections:
    #

    def __check_exchange_existence(self, channel):

        if self.__exchange_name is not None:
            try:
                self.__loginfo(' .. checking exchange ...')
                channel.exchange_declare(self.__exchange_name, passive=True)
                self.__loginfo(' .. checking exchange ... ok.')
            except (pika.exceptions.ChannelClosed) as e:
                self.__loginfo(' .. checking exchange ... failed.')
                self.__add_error_message_no_exchange()
                raise ValueError('The exchange %s does not exist on messaging service host %s' % (self.__exchange_name, self.__current_rabbit_host))
        else:
            pass # No exchange name was given


    def __check_opening_channel(self, connection):
        channel = None
        try:
            channel = self.__open_channel(connection)
            self.__loginfo(' .. checking channel ... ok.')

        except pika.exceptions.ChannelClosed:
            self.__loginfo(' .. checking channel ... FAILED.')
            self.__add_error_message_channel_closed()
            raise ValueError('Channel failed, please try next.')

        return channel

    def __open_channel(self, connection):
        channel = connection.channel()
        channel.confirm_delivery()
        return channel

    def __check_making_rabbit_connection(self):
        connection = None
        try:
            connection = self.__open_rabbit_connection()

        except pika.exceptions.ProbableAuthenticationError:
            self.__loginfo(' .. checking authentication (%s)... FAILED.' % self.__current_rabbit_host)
            self.__add_error_message_authentication_error()
            raise ValueError('Connection failed, please try next.')

        except pika.exceptions.ConnectionClosed:
            self.__loginfo(' .. checking connection (%s)... FAILED.' % self.__current_rabbit_host)
            self.__add_error_message_connection_closed()
            raise ValueError('Connection failed, please try next.')

        if connection is None or not connection.is_open:
            self.__loginfo(' .. checking connection (%s)... FAILED.' % self.__current_rabbit_host)
            self.__add_error_message_connection_problem()
            raise ValueError('Connection failed, please try next.')

        self.__loginfo(' .. checking authentication and connection (%s)... ok.' % self.__current_rabbit_host)
        return connection

    def __open_rabbit_connection(self):
        credentials = pika.PlainCredentials(
            self.__rabbit_username,
            self.__rabbit_password
        )
        params = pika.ConnectionParameters( # https://pika.readthedocs.org/en/0.9.6/connecting.html
            host=self.__current_rabbit_host,
            credentials=credentials
        )
        connection = self.__pika_blocking_connection(params)
        return connection

    def __pika_blocking_connection(self, params): # this is easy to mock
        return pika.BlockingConnection(params)

    #
    # Error messages
    #

    def __add_error_message_general(self):
        self.__error_messages.insert(0,'PROBLEM IN SETTING UP')
        self.__error_messages.insert(1,'RABBIT MESSAGING QUEUE (PID MODULE)')
        self.__error_messages.insert(2, 'CONNECTION TO THE PID MESSAGING QUEUE FAILED DEFINITIVELY:')
        self.__error_messages.append('PLEASE NOTIFY handle@dkrz.de AND INCLUDE THIS ERROR MESSAGE.')

    def __add_error_message_channel_closed(self):
        msg = ' - host "%s": Channel failure.' % self.__current_rabbit_host
        self.__error_messages.append(msg)

    def __add_error_message_no_exchange(self):
        msg = ' - host "%s": Exchange %s does not exist.' % (self.__current_rabbit_host, self.__exchange_name)
        self.__error_messages.append(msg)

    def __add_error_message_authentication_error(self):
        msg = (' - host "%s": Authentication failure (user %s, password %s).' % (
            self.__current_rabbit_host,
            self.__rabbit_username,
            self.__rabbit_password
        ))
        self.__error_messages.append(msg)

    def __add_error_message_connection_closed(self):
        msg = ' - host "%s": Connection failure.' % self.__current_rabbit_host
        self.__error_messages.append(msg)

    def __add_error_message_connection_problem(self):
        msg = ' - host "%s": Unknown connection failure.' % self.__current_rabbit_host
        self.__error_messages.append(msg)

    #
    # Inform at the end
    #

    def __assemble_and_print_error_message(self):
        self.__add_error_message_general()
        error_message_string = utils.format_error_message(self.__error_messages)
        self.__logwarn(error_message_string)

    def __loginfo(self, msg):
        if self.__print_success_to_console == True:
            print(msg)
        utils.loginfo(LOGGER, msg)

    def __logwarn(self, msg):
        if self.__print_errors_to_console == True:
            print(msg)
        utils.logwarn(LOGGER, msg)

    def __define_fallback_exchange(self):
        #print('Called fallback method') # remove
        exchange_name = esgfpid.defaults.RABBIT_FALLBACK_EXCHANGE_NAME
        queue_name = esgfpid.defaults.RABBIT_FALLBACK_EXCHANGE_NAME
        routing_key = esgfpid.defaults.ROUTING_KEY_BASIS
        routing_key = routing_key.replace('HASH', '*')
        routing_key = routing_key+'.#'
        routing_key = routing_key.replace('..', '.')

        # Declare exchange
        try:
            utils.loginfo(LOGGER, 'Declaration of fallback exchange "%s"...' % exchange_name)
            self.channel.exchange_declare(exchange_name, passive=False, durable=True, exchange_type='topic')
            utils.loginfo(LOGGER, 'Declaration of fallback exchange "%s"... done.' % exchange_name)
        except (pika.exceptions.ChannelClosed) as e:
            utils.loginfo(LOGGER, 'Declaration of fallback exchange "%s"... failed. Reasons: %s' % (exchange_name,e))
            self.channel = self.__open_channel(self.connection)

        # Declare queue
        try:
            utils.loginfo(LOGGER, 'Declaration of fallback queue "%s"...' % queue_name)
            self.channel.queue_declare(queue_name, passive=False, durable=True)
            utils.loginfo(LOGGER, 'Declaration of fallback queue "%s"... done.' % queue_name)
        except (pika.exceptions.ChannelClosed) as e:
            utils.loginfo(LOGGER, 'Declaration of fallback queue "%s"... failed. Reasons: %s' % (queue_name,e))
            self.channel = self.__open_channel(self.connection)

        # Bind routing key
        try:
            utils.loginfo(LOGGER, 'Binding of fallback queue with routing key "%s"...' % routing_key)
            self.channel.queue_bind(
                queue = queue_name,
                exchange = exchange_name,
                routing_key = routing_key
            )
            utils.loginfo(LOGGER, 'Binding of fallback queue with routing key "%s"... done.' % routing_key)
        except (pika.exceptions.ChannelClosed) as e:
            utils.loginfo(LOGGER, 'Binding of fallback queue with routing key "%s"... failed. Reasons: %s' % (queue_name,e))




import esgfpid.utils
from esgfpid.defaults import ROUTING_KEY_BASIS as ROUTING_KEY_BASIS

'''
The messages module creates the JSON messages to be sent to the rabbit.
It does not check the type of the values passed. This has to be checked in the assistants,
otherwise it is redundant.
'''

JSON_KEY_ROUTING_KEY = 'ROUTING_KEY'

ROUTING_KEYS = dict(
    publi_file = ROUTING_KEY_BASIS+'publication.file.orig',
    publi_file_rep = ROUTING_KEY_BASIS+'publication.file.replica',
    publi_ds = ROUTING_KEY_BASIS+'publication.dataset.orig',
    publi_ds_rep = ROUTING_KEY_BASIS+'publication.dataset.replica',
    unpubli_all = ROUTING_KEY_BASIS+'unpublication.all',
    unpubli_one = ROUTING_KEY_BASIS+'unpublication.one',
    err_add = ROUTING_KEY_BASIS+'errata.add',
    err_rem = ROUTING_KEY_BASIS+'errata.remove',
    shop_cart = ROUTING_KEY_BASIS+'cart.datasets'
)

def publish_file(**args):

    # Check args:
    mandatory_args = ['file_handle', 'is_replica', 'file_size', 'file_name', 'checksum',
                      'data_url', 'parent_dataset', 'timestamp', 'checksum_type',
                    'file_version', 'data_node']
    esgfpid.utils.check_presence_of_mandatory_args(args, mandatory_args)

    # Message:
    message = dict(
        handle = args['file_handle'],
        aggregation_level = 'file',
        operation = 'publish',
        is_replica=args['is_replica'],
        file_name=args['file_name'],
        file_size=args['file_size'],
        checksum=args['checksum'],
        data_url=args['data_url'],
        data_node=args['data_node'],
        parent_dataset=args['parent_dataset'],
        message_timestamp = args['timestamp'],
        checksum_type = args['checksum_type'],
        file_version = args['file_version'] # can be int or string or ...
    )

    # Routing key:
    routing_key = ROUTING_KEYS['publi_file']
    if args['is_replica'] == True: # Publish Assistant parses this to boolean!
        routing_key = ROUTING_KEYS['publi_file_rep']
    message[JSON_KEY_ROUTING_KEY] = routing_key

    return message

def publish_dataset(**args):

    # Check args:
    mandatory_args = ['dataset_handle', 'drs_id', 'is_replica', 'version_number',
                      'list_of_files', 'data_node', 'timestamp']
    esgfpid.utils.check_presence_of_mandatory_args(args, mandatory_args)

    # Message:
    message = dict(
        handle = args['dataset_handle'],
        aggregation_level = 'dataset',
        operation = 'publish',
        drs_id = args['drs_id'],
        is_replica=args['is_replica'],
        version_number=args['version_number'], # Publish Assistant parses this to int!
        files=args['list_of_files'],
        data_node=args['data_node'],
        message_timestamp = args['timestamp']
    )

    # Optional:
    if 'consumer_solr_url' in args and args['consumer_solr_url'] is not None:
        message['consumer_solr_url'] = args['consumer_solr_url']

    # Routing key:
    routing_key = ROUTING_KEYS['publi_ds']
    if args['is_replica'] == True: # Publish Assistant parses this to boolean!
        routing_key = ROUTING_KEYS['publi_ds_rep']
    message[JSON_KEY_ROUTING_KEY] = routing_key

    return message

def unpublish_allversions_consumer_must_find_versions(**args):

    mandatory_args = ['drs_id', 'data_node', 'timestamp']
    esgfpid.utils.check_presence_of_mandatory_args(args, mandatory_args)

    message = dict(
        operation = 'unpublish_all_versions',
        aggregation_level = 'dataset',
        message_timestamp = args['timestamp'],
        drs_id = args['drs_id'],
        data_node=args['data_node']
    )

    # Optional:
    if 'consumer_solr_url' in args and args['consumer_solr_url'] is not None:
        message['consumer_solr_url'] = args['consumer_solr_url']

    message[JSON_KEY_ROUTING_KEY] = ROUTING_KEYS['unpubli_all']
    return message


def unpublish_one_version(**args):

    mandatory_args = ['data_node', 'timestamp', 'dataset_handle', 'drs_id']
    esgfpid.utils.check_presence_of_mandatory_args(args, mandatory_args)

    message = dict(
        operation = 'unpublish_one_version',
        aggregation_level = 'dataset',
        message_timestamp = args['timestamp'],
        handle = args['dataset_handle'],
        data_node=args['data_node'],
        drs_id = args['drs_id']
    )
    if 'version_number' in args:
        message['version_number'] = args['version_number']

    message[JSON_KEY_ROUTING_KEY] = ROUTING_KEYS['unpubli_one']

    return message

def add_errata_ids_message(**args):

    mandatory_args = ['dataset_handle', 'timestamp', 'errata_ids', 'drs_id', 'version_number']
    esgfpid.utils.check_presence_of_mandatory_args(args, mandatory_args)
  
    message = dict(
        handle = args['dataset_handle'],
        message_timestamp = args['timestamp'],
        errata_ids = args['errata_ids'],
        operation = 'add_errata_ids',
        drs_id = args['drs_id'],
        version_number = args['version_number']
    )

    message[JSON_KEY_ROUTING_KEY] = ROUTING_KEYS['err_add']

    return message

def remove_errata_ids_message(**args):

    mandatory_args = ['dataset_handle', 'timestamp', 'errata_ids', 'drs_id', 'version_number']
    esgfpid.utils.check_presence_of_mandatory_args(args, mandatory_args)
    
    message = dict(
        handle = args['dataset_handle'],
        message_timestamp = args['timestamp'],
        errata_ids = args['errata_ids'],
        operation = 'remove_errata_ids',
        drs_id = args['drs_id'],
        version_number = args['version_number']
    )
    
    message[JSON_KEY_ROUTING_KEY] = ROUTING_KEYS['err_rem']

    return message

def make_shopping_cart_message(**args):
    mandatory_args = ['cart_handle', 'timestamp', 'data_cart_content']
    esgfpid.utils.check_presence_of_mandatory_args(args, mandatory_args)
    message = dict(
        handle = args['cart_handle'],
        message_timestamp = args['timestamp'],
        data_cart_content = args['data_cart_content'],
        operation = 'shopping_cart'
    )
    message[JSON_KEY_ROUTING_KEY] = ROUTING_KEYS['shop_cart']
    return message


import unittest
import mock
import logging
import json
import sys
sys.path.append("..")
import esgfpid.rabbit
import tests.mocks.pikamock
import pika.exceptions
import pika

LOGGER = logging.getLogger(__name__)
LOGGER.addHandler(logging.NullHandler())

class RabbitConnectorTestCase(unittest.TestCase):

    def setUp(self):
        LOGGER.info('######## Next test ##########')

    def tearDown(self):
        LOGGER.info('#############################')

    def __get_args_dict(self):

        cred = esgfpid.rabbit.connparams.get_credentials('johndoe','pass123')
        args = dict(
            exchange_name = 'exch',
            url_preferred = 'first.host',
            urls_fallback = ['fallback.host', 'fallback.host.too'],
            credentials = cred
        )
        return args

    # Tests

    #
    # Setup of the connection/channel:
    #

    def test_init_ok(self):

        # Test variables:
        args = self.__get_args_dict()

        # Run code to be tested:
        testrabbit = esgfpid.rabbit.synchronous.SynchronousServerConnector(**args)

        # Check result
        self.assertIsInstance(testrabbit, esgfpid.rabbit.synchronous.SynchronousServerConnector)
        # Communcation not established yet:
        established = testrabbit._SynchronousServerConnector__communication_established
        self.assertFalse(established)

    @mock.patch('esgfpid.rabbit.synchronous.SynchronousServerConnector._SynchronousServerConnector__make_connection')
    def test_open_rabbit_connection_ok(self, connectionmock):

        # Define replacement for mocked object:
        def make_mocked_connection(params):
            return tests.mocks.pikamock.MockPikaBlockingConnection(params)
        connectionmock.side_effect = make_mocked_connection

        # Make test rabbit:
        args = self.__get_args_dict()
        testrabbit = esgfpid.rabbit.synchronous.SynchronousServerConnector(**args)
        
        # Run code to be tested:
        testrabbit.open_rabbit_connection()

        # Check result:
        # Connection created:
        conn = testrabbit._SynchronousServerConnector__connection
        self.assertIsInstance(conn, tests.mocks.pikamock.MockPikaBlockingConnection)
        # Channel created:
        channel = testrabbit._SynchronousServerConnector__channel
        self.assertIsInstance(channel, tests.mocks.pikamock.MockChannel)
        # Communcation established:
        established = testrabbit._SynchronousServerConnector__communication_established
        self.assertTrue(established)
        # Connected to preferred URL:
        expected_hosts = args['url_preferred']
        self.assertIn(conn.host, expected_hosts, 'Connected to: %s' % conn.host)

    @mock.patch('esgfpid.rabbit.synchronous.SynchronousServerConnector._SynchronousServerConnector__make_connection')
    def test_open_rabbit_connection_not_ok(self, connectionmock):
        '''
        All URLs will cause ProbableAuthenticationErrors,
        so no connection can be opened.
        '''

        # Define replacement for mocked object:
        connectionmock.side_effect = pika.exceptions.AuthenticationError

        # Make test rabbit:
        args = self.__get_args_dict()
        testrabbit = esgfpid.rabbit.synchronous.SynchronousServerConnector(**args)
        
        # Run code to be tested:
        testrabbit.open_rabbit_connection()

        # Check result:
        # Connection is None:
        conn = testrabbit._SynchronousServerConnector__connection
        self.assertIsNone(conn)
        # Channel is None:
        channel = testrabbit._SynchronousServerConnector__channel
        self.assertIsNone(channel)
        # Communcation not established:
        established = testrabbit._SynchronousServerConnector__communication_established
        self.assertFalse(established)

    @mock.patch('esgfpid.rabbit.synchronous.SynchronousServerConnector._SynchronousServerConnector__make_connection')
    def test_open_rabbit_connection_first_fails_auth(self, connectionmock):
        '''
        The first URL causes a ProbableAuthenticationError,
        the fallback URL works, so the connection will be ok.
        '''

        # Define replacement for mocked object:
        def make_mocked_connection(params):
            if params.host == 'first.host':
                raise pika.exceptions.ProbableAuthenticationError
            else:
                return tests.mocks.pikamock.MockPikaBlockingConnection(params)
        connectionmock.side_effect = make_mocked_connection

        # Make test rabbit:
        args = self.__get_args_dict()
        testrabbit = esgfpid.rabbit.synchronous.SynchronousServerConnector(**args)

        # Run code to be tested:
        testrabbit.open_rabbit_connection()

        # Check result:
        # Connection created:
        conn = testrabbit._SynchronousServerConnector__connection
        self.assertIsInstance(conn, tests.mocks.pikamock.MockPikaBlockingConnection)
        # Channel created:
        channel = testrabbit._SynchronousServerConnector__channel
        self.assertIsInstance(channel, tests.mocks.pikamock.MockChannel)
        # Communcation established:
        established = testrabbit._SynchronousServerConnector__communication_established
        self.assertTrue(established)
        # Connected to fallback URL:
        expected_hosts = args['urls_fallback']
        self.assertIn(conn.host, expected_hosts, 'Connected to: %s' % conn.host)


    @mock.patch('esgfpid.rabbit.synchronous.SynchronousServerConnector._SynchronousServerConnector__make_connection')
    def test_open_rabbit_connection_first_fails_conn(self, connectionmock):
        '''
        The first URL causes a AMQPConnectionError,
        the fallback URL works, so the connection will be ok.
        '''

        # Define replacement for mocked object:
        def make_mocked_connection(params):
            if params.host == 'first.host':
                raise pika.exceptions.AMQPConnectionError
            else:
                return tests.mocks.pikamock.MockPikaBlockingConnection(params)
        connectionmock.side_effect = make_mocked_connection

        # Make test rabbit:
        args = self.__get_args_dict()
        testrabbit = esgfpid.rabbit.synchronous.SynchronousServerConnector(**args)

        # Run code to be tested:
        testrabbit.open_rabbit_connection()

        # Check result:
        # Connection created:
        conn = testrabbit._SynchronousServerConnector__connection
        self.assertIsInstance(conn, tests.mocks.pikamock.MockPikaBlockingConnection)
        # Channel created:
        channel = testrabbit._SynchronousServerConnector__channel
        self.assertIsInstance(channel, tests.mocks.pikamock.MockChannel)
        # Communcation established:
        established = testrabbit._SynchronousServerConnector__communication_established
        self.assertTrue(established)
        # Connected to fallback URL:
        expected_hosts = args['urls_fallback']
        self.assertIn(conn.host, expected_hosts, 'Connected to: %s' % conn.host)

    @mock.patch('esgfpid.rabbit.synchronous.SynchronousServerConnector._SynchronousServerConnector__make_connection')
    def test_open_rabbit_connection_first_two_fail(self, connectionmock):
        '''
        The first two URLs cause a AMQPConnectionError,
        the last fallback URL works, so the connection will be ok.
        '''

        # Define replacement for mocked object:
        def make_mocked_connection(params):
            if params.host == 'first.host' or params.host == 'fallback.host.too':
                raise pika.exceptions.AMQPConnectionError
            else:
                return tests.mocks.pikamock.MockPikaBlockingConnection(params)
        connectionmock.side_effect = make_mocked_connection

        # Make test rabbit:
        args = self.__get_args_dict()
        testrabbit = esgfpid.rabbit.synchronous.SynchronousServerConnector(**args)

        # Run code to be tested:
        testrabbit.open_rabbit_connection()

        # Check result:
        # Connection created:
        conn = testrabbit._SynchronousServerConnector__connection
        self.assertIsInstance(conn, tests.mocks.pikamock.MockPikaBlockingConnection)
        # Channel created:
        channel = testrabbit._SynchronousServerConnector__channel
        self.assertIsInstance(channel, tests.mocks.pikamock.MockChannel)
        # Communcation established:
        established = testrabbit._SynchronousServerConnector__communication_established
        self.assertTrue(established)
        # Connected to fallback URL:
        expected_hosts = args['urls_fallback']
        self.assertIn(conn.host, expected_hosts, 'Connected to: %s' % conn.host)

    @mock.patch('esgfpid.rabbit.synchronous.SynchronousServerConnector._SynchronousServerConnector__make_channel')
    @mock.patch('esgfpid.rabbit.synchronous.SynchronousServerConnector._SynchronousServerConnector__make_connection')
    def test_open_rabbit_channel_fails(self, connectionmock, channelmock):

        # Define replacement for mocked object:
        # Mock connection:
        def make_mocked_connection(params):
            return tests.mocks.pikamock.MockPikaBlockingConnection(params)
        connectionmock.side_effect = make_mocked_connection
        # Mock channel:
        channelmock.side_effect = pika.exceptions.ChannelClosed

        # Make test rabbit:
        args = self.__get_args_dict()
        testrabbit = esgfpid.rabbit.synchronous.SynchronousServerConnector(**args)
        
        # Run code to be tested:
        testrabbit.open_rabbit_connection()

        # Check result:
        # Connection created:
        conn = testrabbit._SynchronousServerConnector__connection
        self.assertIsInstance(conn, tests.mocks.pikamock.MockPikaBlockingConnection)
        # Channel is None:
        channel = testrabbit._SynchronousServerConnector__channel
        self.assertIsNone(channel)
        # Communcation NOT established:
        established = testrabbit._SynchronousServerConnector__communication_established
        self.assertFalse(established)
        # Connected to fallback URL:
        expected_hosts = args['urls_fallback']
        self.assertIn(conn.host, expected_hosts, 'Connected to: %s' % conn.host)

    #
    # Close connection
    #

    @mock.patch('esgfpid.rabbit.synchronous.SynchronousServerConnector._SynchronousServerConnector__make_connection')
    def test_close_rabbit_connection_ok(self, connectionmock):

        # Define replacement for mocked object:
        def make_mocked_connection(params):
            return tests.mocks.pikamock.MockPikaBlockingConnection(params)
        connectionmock.side_effect = make_mocked_connection

        # Make test rabbit with connection:
        args = self.__get_args_dict()
        testrabbit = esgfpid.rabbit.synchronous.SynchronousServerConnector(**args)
        testrabbit.open_rabbit_connection()

        # Run code to be tested:
        testrabbit.close_rabbit_connection()

        # Check result:
        # Connection should be closed:
        conn = testrabbit._SynchronousServerConnector__connection
        self.assertFalse(conn.is_open)

    #
    # Send message
    #

    @mock.patch('esgfpid.rabbit.synchronous.SynchronousServerConnector._SynchronousServerConnector__make_connection')
    def test_send_message_ok(self, connectionmock):
        '''
        We expect the message to arrive in the MockChannel
        on the first try.
        '''

        # Test variables:
        key = 'mykey'
        msg = {"foo":"bar", "ROUTING_KEY":key}

        # Define replacement for mocked object:
        def make_mocked_connection(params):
            return tests.mocks.pikamock.MockPikaBlockingConnection(params)
        connectionmock.side_effect = make_mocked_connection

        # Make test rabbit with connection:
        args = self.__get_args_dict()
        testrabbit = esgfpid.rabbit.synchronous.SynchronousServerConnector(**args)
        testrabbit.open_rabbit_connection()

        # Run code to be tested:
        testrabbit.send_message_to_queue(msg)

        # Check result:
        # Message was sent:
        channel = testrabbit._SynchronousServerConnector__channel
        self.assertEquals(channel.success_counter, 1, 'Message should succeed!')
        # Message was sent on first try:
        self.assertEqual(channel.publish_counter, 1,
            'Message was sent %i times!' % channel.publish_counter)
        # Message arrived in mock channel:
        self.assertIn(json.dumps(msg), channel.messages,
            'Message %s not found in channel\'s list: %s' % (msg, channel.messages))
        self.assertIn(key, channel.routing_keys,
            'Routing key %s not found in channel\'s list: %s' % (key, channel.routing_keys))


    @mock.patch('esgfpid.rabbit.synchronous.SynchronousServerConnector._SynchronousServerConnector__make_connection')
    def test_send_message_fail(self, connectionmock):

        # Test variables:
        key = 'mykey'
        msg = {"foo":"bar", "ROUTING_KEY":key}

        # Define replacement for mocked function:
        def make_mocked_connection(params):
            return tests.mocks.pikamock.MockPikaBlockingConnection(params)
        connectionmock.side_effect = make_mocked_connection

        # Make test rabbit with connection:
        args = self.__get_args_dict()
        testrabbit = esgfpid.rabbit.synchronous.SynchronousServerConnector(**args)
        testrabbit.open_rabbit_connection()

        # Make publish raise an UnroutableError
        channel = testrabbit._SynchronousServerConnector__channel
        channel.num_failures = float('inf') # always failure!

        # Run code to be tested:
        testrabbit.send_message_to_queue(msg)

        # Check result:
        self.assertEquals(channel.success_counter, 0, 'Message should not succeed!')



    @mock.patch('esgfpid.rabbit.synchronous.SynchronousServerConnector._SynchronousServerConnector__make_connection')
    def test_send_message_unroutable(self, connectionmock):

        # Test variables:
        key = 'mykey'
        msg = {"foo":"bar", "ROUTING_KEY":key}

        # Define replacement for mocked object:
        def make_mocked_connection(params):
            return tests.mocks.pikamock.MockPikaBlockingConnection(params)
        connectionmock.side_effect = make_mocked_connection
        # Make basic_publish raise an error
        #publishmock.side_effect = pika.exceptions.UnroutableError('foo bar')

        # Make test rabbit with connection:
        args = self.__get_args_dict()
        testrabbit = esgfpid.rabbit.synchronous.SynchronousServerConnector(**args)
        testrabbit.open_rabbit_connection()

        # Make publish raise an UnroutableError
        channel = testrabbit._SynchronousServerConnector__channel
        channel.num_unroutables = float('inf') # always unroutable!

        # Run code to be tested:
        testrabbit.send_message_to_queue(msg)

        # Check result:
        self.assertEquals(channel.success_counter, 0, 'Message should not succeed!')

    @mock.patch('esgfpid.rabbit.synchronous.SynchronousServerConnector._SynchronousServerConnector__make_connection')
    def test_send_message_first_fails_then_ok(self, connectionmock):
        '''
        We make the channel return False (failure), then True (success).
        We check if the message arrived in the MockChannel.
        '''

        # Test variables:
        key = 'mykey'
        msg = {"foo":"bar", "ROUTING_KEY":key}

        # Define replacement for mocked object:
        def make_mocked_connection(params):
            return tests.mocks.pikamock.MockPikaBlockingConnection(params)
        connectionmock.side_effect = make_mocked_connection

        # Make test rabbit with connection:
        args = self.__get_args_dict()
        testrabbit = esgfpid.rabbit.synchronous.SynchronousServerConnector(**args)
        testrabbit.open_rabbit_connection()

        # Make publish succeed only in 2nd try:
        channel = testrabbit._SynchronousServerConnector__channel
        channel.num_failures = 1

        # Run code to be tested:
        testrabbit.send_message_to_queue(msg)

        # Check result:
        # Message was sent:
        self.assertEquals(channel.success_counter, 1, 'Message should succeed!')
        # Message was sent on second try:
        self.assertEqual(channel.publish_counter, 2,
            'Message was sent %i times!' % channel.publish_counter)
        # Message arrived in mock channel:
        self.assertIn(json.dumps(msg), channel.messages,
            'Message %s not found in channel\'s list: %s' % (msg, channel.messages))
        self.assertIn(key, channel.routing_keys,
            'Routing key %s not found in channel\'s list: %s' % (key, channel.routing_keys))

    @mock.patch('esgfpid.rabbit.synchronous.SynchronousServerConnector._SynchronousServerConnector__make_connection')
    def test_send_message_no_connection(self, connectionmock):
        '''
        If there is no connection, it is built before publishing
        the message.
        We check if the message arrived in the MockChannel.
        '''

        # Test variables:
        key = 'mykey'
        msg = {"foo":"bar", "ROUTING_KEY":key}

        # Define replacement for mocked object:
        def make_mocked_connection(params):
            return tests.mocks.pikamock.MockPikaBlockingConnection(params)
        connectionmock.side_effect = make_mocked_connection

        # Make test rabbit without connection:
        args = self.__get_args_dict()
        testrabbit = esgfpid.rabbit.synchronous.SynchronousServerConnector(**args)

        # Run code to be tested:
        testrabbit.send_message_to_queue(msg)

        # Check result:
        # Connection is opened:
        conn = testrabbit._SynchronousServerConnector__connection
        self.assertTrue(conn.is_open)
        # Message was sent:
        channel = testrabbit._SynchronousServerConnector__channel
        self.assertEquals(channel.success_counter, 1, 'Message should succeed!')
        # Message was sent on first try:
        self.assertEqual(channel.publish_counter, 1,
            'Message was sent %i times!' % channel.publish_counter)
        # Message arrived in mock channel:
        self.assertIn(json.dumps(msg), channel.messages,
            'Message %s not found in channel\'s list: %s' % (msg, channel.messages))
        self.assertIn(key, channel.routing_keys,
            'Routing key %s not found in channel\'s list: %s' % (key, channel.routing_keys))